@@ -26,9 +26,6 @@
  *   2. Myers bit vector (describes vertical (horizontal) tiling)
  *     Gene Myers, A fast bit-vector algorithm for approximate string matching based on dynamic programming, JACM (1999)
  */
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 /* make sure POSIX APIs are properly activated */
 #if defined(__linux__) && !defined(_POSIX_C_SOURCE)
@@ -39,12 +36,19 @@ extern "C" {
 #  define _BSD_SOURCE
 #endif
 
+#define SIMDE_ENABLE_NATIVE_ALIASES
+#include "simde/x86/sse4.1.h"
+
+// Only turn on extern "C" after including code that can use C++ features.
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <assert.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <stddef.h>
 #include <string.h>
-#include <x86intrin.h>
 
 #ifndef DZ_INCLUDE_ONCE
 
@@ -79,14 +83,20 @@ enum dz_alphabet {
 		rN = 0x90, qN = 0x90, qS = 0x02		/* pshufb instruction clears the column when the 7-th bit of the index is set */
 	#endif
 };
+#ifdef DZ_FULL_LENGTH_BONUS
+#define dz_end_bonus(_self, _query, _i)            ((_i) / 8 == (_query)->blen - 1 ? (_query)->bonus[8 + ((_i) & 7)] : 0)
+#else
+#define dz_end_bonus(_self, _query, _i)            0;
+#endif
+
 /* get the first index of the quals from a packed query (only valid if DZ_QUAL_ADJ is defined) */
 #define dz_quals(_query)                           ( (uint8_t const *) &(_query)->arr[(_query)->arr_end] )
 /* get the base quality adjusted matrix (only valid if DZ_QUAL_ADJ is defined) */
 #define dz_qual_matrix(_self)                      ( (int8_t const *)((_self) + 1) )
 
-#define dz_pair_score(_self, _q, _r, _i)	       ( (_self)->matrix[((_r) | (_q)->arr[(_i)]) & 0x1f] )
+#define dz_pair_score(_self, _q, _r, _i)	       ( (_self)->matrix[((_r) | (_q)->arr[(_i)]) & 0x1f] + dz_end_bonus(_self, _q, _i))
 
-#define dz_qual_adj_pair_score(_self, _q, _r, _i)  ( dz_qual_matrix(_self)[(((uint32_t) dz_quals(_q)[(_i)]) << 5) | ((uint32_t)(_r)) | ((uint32_t)((_q)->arr[(_i)] & 0x1f))] )
+#define dz_qual_adj_pair_score(_self, _q, _r, _i)  ( dz_qual_matrix(_self)[(((uint32_t) dz_quals(_q)[(_i)]) << 5) | ((uint32_t)(_r)) | ((uint32_t)((_q)->arr[(_i)] & 0x1f))] + dz_end_bonus(_self, _q, _i) )
 
 #define dz_pair_eq(_self, _q, _r, _i)		       ( (uint32_t)((_q)->arr[(_i)]) == ((uint32_t)(_r)<<2) )
 
@@ -96,7 +106,7 @@ enum dz_alphabet {
 #  ifndef DZ_MAT_SIZE
 #    define DZ_MAT_SIZE				( 32 )
 #  endif
-#define dz_pair_score(_self, _q, _r, _i)	( (int8_t)((_q)->arr[(_r) * (_q)->blen * L + (_i)]) )
+#define dz_pair_score(_self, _q, _r, _i)	( (int8_t)((_q)->arr[(_r) * (_q)->blen * L + (_i)]) + dz_end_bonus(_self, _q, _i))
 #define dz_pair_eq(_self, _q, _r, _i)		( (uint32_t)((_q)->q[(_i) - 1] & 0x1f) == (uint32_t)(_r) )
 #endif
 
@@ -139,15 +149,7 @@ unittest() { debug("hello"); }
 #define dz_ut_sel(a, b, c)			( (DZ_UNITTEST_INDEX == 0) ? (a) : ((DZ_UNITTEST_INDEX == 1) ? (b) : (c)) )
 
 /* vectorize */
-#ifndef __x86_64__
-#  error "x86_64 is required"
-#endif
-#ifndef __SSE4_1__
-#  warning "SSE4.1 is automatically enabled in dozeu.h, please check compatibility to the system."
-#  define __dz_vectorize			__attribute__(( target( "sse4.1" ) ))
-#else
-#  define __dz_vectorize			/* follow the compiler options */
-#endif
+#define __dz_vectorize			/* follow the compiler options */
 
 /* inlining (FIXME: add appropriate __force_inline flag) */
 #define __dz_force_inline			inline
@@ -166,11 +168,7 @@ unittest() { debug("hello"); }
 #define dz_loadu_u64(p)				({ uint8_t const *_p = (uint8_t const *)(p); *((uint64_t const *)_p); })
 #define dz_storeu_u64(p, e)			{ uint8_t *_p = (uint8_t *)(p); *((uint64_t *)(_p)) = (e); }
 
-#ifdef __SSE4_1__
-#  define dz_is_all_zero(x)			( _mm_test_all_zeros((x), (x)) == 1 )
-#else
-#  define dz_is_all_zero(x)			( _mm_movemask_epi8((x)) == 0 )
-#endif
+#define dz_is_all_zero(x)			( _mm_test_all_zeros((x), (x)) == 1 )
 
 #define DZ_MEM_MARGIN_SIZE			( 256 )
 #define DZ_MEM_ALIGN_SIZE			( 16 )
@@ -256,7 +254,7 @@ struct dz_mem_s { struct dz_mem_block_s blk; struct dz_stack_s stack; };
 
 struct dz_s {
     int8_t matrix[32];
-    uint16_t giv[8], gev[8], riv[8], rev[8], bonus, _pad[7]; // padding ensures memory alignment
+    uint16_t giv[8], gev[8], riv[8], rev[8]; // padding ensures memory alignment
     int8_t protein_matrix[];
 };
 dz_static_assert(sizeof(struct dz_s) % sizeof(__m128i) == 0);
@@ -642,7 +640,7 @@ unittest() {
 
                      
 #define _update_vector(_p) { \
-	__m128i sc = _calc_score_profile(_p); \
+	__m128i sc = _add_bonus(_p, _calc_score_profile(_p)); \
 	__m128i te = _mm_subs_epi16(_mm_max_epi16(e, _mm_subs_epi16(s, giv)), gev1); \
 	/* print_vector(_mm_alignr_epi8(s, ps, 14)); print_vector(sc); */ \
 	__m128i ts = _mm_max_epi16(te, _mm_adds_epi16(sc, _mm_alignr_epi8(s, ps, 14))); \
@@ -652,7 +650,7 @@ unittest() {
 	tf = _mm_max_epi16(tf, _mm_subs_epi16(_mm_alignr_epi8(tf, minv, 12), gev2)); \
 	tf = _mm_max_epi16(tf, _mm_subs_epi16(_mm_alignr_epi8(tf, minv, 8), gev4)); \
 	ts = _mm_max_epi16(ts, tf); print_vector(ts); \
-	maxv = _mm_max_epi16(maxv, _add_bonus(_p, ts)); \
+	maxv = _mm_max_epi16(maxv, ts); \
 	/* print_vector(te); print_vector(_add_bonus(_p, ts)); print_vector(tf); print_vector(maxv);*/ \
 	e = te; f = tf; s = ts; \
     pts = ts; \
@@ -688,9 +686,7 @@ struct dz_s *dz_init_intl(
     int8_t const *qual_adj_score_matrix, /* series of DZ_NUM_QUAL_SCORES adjusted score matrices for each phred base qual (no offset) */
 #endif
 	uint16_t gap_open,				/* gap penalties in positive */
-	uint16_t gap_extend,
-	//uint64_t max_gap_len,			/* as X-drop threshold */
-	uint16_t full_length_bonus)		/* end-to-end mapping bonus; only activated when compiled with -DDZ_FULL_LENGTH_BONUS */
+	uint16_t gap_extend)
 {
 	/*
 	static uint8_t const transpose[16] __attribute__(( aligned(16) )) = {
@@ -744,17 +740,8 @@ struct dz_s *dz_init_intl(
 	__m128i const gev = _mm_set1_epi16(gap_extend);
 	_mm_store_si128((__m128i *)self->giv, giv);
 	_mm_store_si128((__m128i *)self->gev, gev);
-    // TODO: see if i can remove this to cut down on state
-	//self->xt = 0;			/* X-drop threshold */
-	self->bonus = full_length_bonus;
-	//self->max_gap_len = max_gap_len;			/* save raw value */
-	debug("gi(%u), ge(%u), full_length_bonus(%u)", gap_open, gap_extend, self->bonus);
+	debug("gi(%u), ge(%u)", gap_open, gap_extend);
 
-    // TODO: i'm pretty sure this cap is extraneous and never used
-	///* create root head */
-	//struct dz_cap_s *cap = (struct dz_cap_s *)dz_mem_malloc(mem, sizeof(struct dz_cap_s));
-	//_mm_store_si128((__m128i *)cap, _mm_setzero_si128());
-    
     /* initialize and memoize the vectors we need to compute the root */
 	__m128i riv = _mm_setr_epi16(
         0,
@@ -797,14 +784,12 @@ struct dz_s *dz_init(
     int8_t const *qual_adj_score_matrix, /* series of DZ_NUM_QUAL_SCORES adjusted score matrices for each phred base qual (no offset) */
 #endif
 	uint16_t gap_open,
-	uint16_t gap_extend,
-	//uint64_t max_gap_len,
-	uint16_t full_length_bonus)
+	uint16_t gap_extend)
 {
 #ifdef DZ_QUAL_ADJ
-    return(dz_qual_adj_init_intl(score_matrix, qual_adj_score_matrix, gap_open, gap_extend, full_length_bonus));
+    return(dz_qual_adj_init_intl(score_matrix, qual_adj_score_matrix, gap_open, gap_extend));
 #else
-	return(dz_init_intl(score_matrix, gap_open, gap_extend, full_length_bonus));
+	return(dz_init_intl(score_matrix, gap_open, gap_extend));
 #endif
 }
 
@@ -1023,7 +1008,7 @@ static int8_t const dz_unittest_score_matrix[DZ_MAT_SIZE * DZ_MAT_SIZE] = {
 
 #ifndef DZ_UNITTEST_SCORE_PARAMS
 #ifdef DZ_FULL_LENGTH_BONUS
-#define DZ_UNITTEST_SCORE_PARAMS	dz_unittest_score_matrix, 5, 1, 0
+#define DZ_UNITTEST_SCORE_PARAMS	dz_unittest_score_matrix, 5, 1
 #else
 #define DZ_UNITTEST_SCORE_PARAMS	dz_unittest_score_matrix, 5, 1
 #endif
@@ -1062,7 +1047,10 @@ struct dz_query_s *dz_pack_query_forward(
 #ifdef DZ_QUAL_ADJ
     uint8_t const *qual,
 #endif
-	 size_t qlen)
+#ifdef DZ_FULL_LENGTH_BONUS
+    int8_t full_length_bonus,
+#endif
+    size_t qlen)
 {
 	size_t const L = sizeof(__m128i) / sizeof(uint16_t);
     // TODO: qual: make sure I'm doing this right
@@ -1082,8 +1070,9 @@ struct dz_query_s *dz_pack_query_forward(
     /*
 	 * tentative support of full-length bonus; precaclulated bonus score is saved at q->bonus[0] and q->bonus[1]; [0] for non-tail vector and [1] for tail vector
 	 */
-	q->bonus[L + (qlen % L)] = self->bonus;
-
+    #ifdef DZ_FULL_LENGTH_BONUS
+        q->bonus[L + (qlen % L)] = full_length_bonus;
+    #endif
 	/*
 	 * ASCII to 2-bit conversion table (shifted by two bits to be used as the upper (row) shuffle index)
 	 * @ABC_DEFG_HIJK_LMNO
@@ -1156,6 +1145,9 @@ struct dz_query_s *dz_pack_query_reverse(
     char const *query,
 #ifdef DZ_QUAL_ADJ
     uint8_t const *qual,
+#endif
+#ifdef DZ_FULL_LENGTH_BONUS
+    int8_t full_length_bonus,
 #endif
     size_t qlen)
 {
@@ -1173,7 +1165,9 @@ struct dz_query_s *dz_pack_query_reverse(
 		.q = query,
 		.bonus = { 0 }
 	};
-	q->bonus[L + (qlen % L)] = self->bonus;
+    #ifdef DZ_FULL_LENGTH_BONUS
+        q->bonus[L + (qlen % L)] = full_length_bonus;
+    #endif
 
 	static uint8_t const conv[16] __attribute__(( aligned(16) )) = {
 		#ifdef DZ_NUCL_ASCII
@@ -1236,7 +1230,11 @@ struct dz_query_s *dz_pack_query_reverse(
 static __dz_vectorize
 struct dz_query_s *dz_pack_query_forward(
 	struct dz_s *self,
-	char const *query, size_t qlen)
+	char const *query,
+#ifdef DZ_FULL_LENGTH_BONUS
+    int8_t full_length_bonus,
+#endif
+    size_t qlen)
 {
 	size_t const L = sizeof(__m128i) / sizeof(uint16_t);
 	struct dz_query_s *q = (struct dz_query_s *)dz_mem_malloc(dz_mem(self), sizeof(struct dz_query_s) + DZ_MAT_SIZE * dz_roundup(qlen + 1, L) + sizeof(__m128i));
@@ -1245,7 +1243,10 @@ struct dz_query_s *dz_pack_query_forward(
 		.q = query,
 		.bonus = { 0 }
 	};
-	q->bonus[L + (qlen % L)] = self->bonus;
+    
+    #ifdef DZ_FULL_LENGTH_BONUS
+        q->bonus[L + (qlen % L)] = full_length_bonus;
+    #endif
 
 	for(uint64_t j = 0; j < DZ_MAT_SIZE; j++) {
 		size_t const clen = dz_roundup(qlen + 1, L);
@@ -1282,7 +1283,11 @@ struct dz_query_s *dz_pack_query_forward(
 static __dz_vectorize
 struct dz_query_s *dz_pack_query_reverse(
 	struct dz_s *self,
-	char const *query, size_t qlen)
+	char const *query,
+#ifdef DZ_FULL_LENGTH_BONUS
+    int8_t full_length_bonus,
+#endif
+    size_t qlen)
 {
 	size_t const L = sizeof(__m128i) / sizeof(uint16_t);
 	struct dz_query_s *q = (struct dz_query_s *)dz_mem_malloc(dz_mem(self), sizeof(struct dz_query_s) + DZ_MAT_SIZE * dz_roundup(qlen + 1, L) + sizeof(__m128i));
@@ -1291,7 +1296,9 @@ struct dz_query_s *dz_pack_query_reverse(
 		.q = query,
 		.bonus = { 0 }
 	};
-	q->bonus[L + (qlen % L)] = self->bonus;
+    #ifdef DZ_FULL_LENGTH_BONUS
+        q->bonus[L + (qlen % L)] = full_length_bonus;
+    #endif
 
 	for(uint64_t j = 0; j < DZ_MAT_SIZE; j++) {
 		size_t const clen = dz_roundup(qlen + 1, L);
@@ -1342,20 +1349,23 @@ struct dz_query_s *dz_pack_query(
 	char const *query,
 #ifdef DZ_QUAL_ADJ
     const uint8_t *qual,
+#endif
+#ifdef DZ_FULL_LENGTH_BONUS
+    int8_t full_length_bonus,
 #endif
     int64_t qlen)
 {
 	if(qlen >= 0) {
 #ifdef DZ_QUAL_ADJ
-        return(dz_qual_adj_pack_query_forward(self, query, qual, (size_t)qlen));
+        return(dz_qual_adj_pack_query_forward(self, query, qual, full_length_bonus, (size_t)qlen));
 #else
-        return(dz_pack_query_forward(self, query, (size_t)qlen));
+        return(dz_pack_query_forward(self, query, full_length_bonus, (size_t)qlen));
 #endif
 	} else {
 #ifdef DZ_QUAL_ADJ
-        return(dz_qual_adj_pack_query_reverse(self, &query[qlen], &qual[qlen], (size_t)-qlen));
+        return(dz_qual_adj_pack_query_reverse(self, &query[qlen], &qual[qlen], full_length_bonus, (size_t)-qlen));
 #else
-        return(dz_pack_query_reverse(self, &query[qlen], (size_t)-qlen));
+        return(dz_pack_query_reverse(self, &query[qlen], full_length_bonus, (size_t)-qlen));
 #endif
 	}
 }
@@ -1366,7 +1376,12 @@ unittest() {
 	struct dz_s *dz = dz_init(DZ_UNITTEST_SCORE_PARAMS);
 	ut_assert(dz != NULL);
 
-	struct dz_query_s *q = dz_pack_query(dz, dz_unittest_query, dz_unittest_query_length);
+	struct dz_query_s *q = dz_pack_query(dz, dz_unittest_query,
+#ifdef DZ_FULL_LENGTH_BONUS
+                                         0,
+#endif
+                                         
+                                         dz_unittest_query_length);
 	dz_unused(q);
 	dz_destroy(dz);
 }
@@ -1426,20 +1441,29 @@ unittest() {
 	__m128i const xtv = _mm_set1_epi16(w.inc - xt);	/* next offset == current max thus X-drop threshold is always -xt */ \
 	/* until the bottommost vertically placed band... */ \
 	uint32_t sspos = w.r.spos;					/* save spos on the stack */ \
+    uint64_t first_drop = w.r.epos;\
 	for(uint64_t p = w.r.spos; p < w.r.epos; p++) { \
 		_load_vector(&pdp[p]); \
         _update_vector(p); \
 		if(dz_unlikely(_test_xdrop(s, xtv))) {	/* mark _unlikely to move out of the core loop */ \
 			/* drop either leading or trailing vector, skip the forefront extension when the forefront is clipped */ \
+            pts = _mm_set1_epi16(INT16_MIN); \
 			if(p == w.r.spos) { \
                 w.r.spos++; cdp--; continue; \
             } else { \
-                 w.r.epos = p; goto dz_pp_cat(_forefront_, __LINE__); \
+                first_drop = dz_min2(p, first_drop); \
             } \
-		} \
+        } else { \
+            first_drop = w.r.epos; /* reset the xdrop to indicate that any xdrops we found are not contiguous */ \
+        } \
 		_store_vector(&cdp[p]); \
 	} \
-	/* if reached the forefront of the query sequence, finish the extension */ \
+    if (first_drop < w.r.epos) { \
+        /* we xdropped a contiguous block of vectors at the end of this column */\
+        w.r.epos = first_drop; \
+        goto dz_pp_cat(_forefront_, __LINE__); \
+    } \
+    /* if reached the forefront of the query sequence, finish the extension */ \
 	if(w.r.epos < query->blen) { \
 		/* forefront extension; clip the column length if too long */ \
 		__m128i e = minv, s = minv; _update_vector(w.r.epos); \
@@ -1590,6 +1614,9 @@ unittest( "extend.base" ) {
 
 	for(size_t trial = 0; trial < 3; trial++) {
 		struct dz_query_s const *q = dz_pack_query(dz, dz_unittest_query,
+#ifdef DZ_FULL_LENGTH_BONUS
+                                                   0,
+#endif
 			  trial == 0 ? dz_unittest_query_length
 			: trial == 1 ? 3
 			:              10
@@ -1649,7 +1676,11 @@ unittest( "extend.base.revcomp" ) {
 		size_t length = trial == 0 ? dz_unittest_query_length
 					  : trial == 1 ? 3
 					  :              10;
-		struct dz_query_s const *q = dz_pack_query_reverse(dz, &revcomp[dz_unittest_query_length - length], length);
+		struct dz_query_s const *q = dz_pack_query_reverse(dz, &revcomp[dz_unittest_query_length - length],
+#ifdef DZ_FULL_LENGTH_BONUS
+                                                           0,
+#endif
+                                                           length);
 		struct dz_forefront_s const *forefront = NULL;
 
 		/* nothing occurs */
@@ -1699,6 +1730,9 @@ unittest( "extend.small" ) {
 
 	for(size_t trial = 0; trial < 4; trial++) {
 		struct dz_query_s const *q = dz_pack_query(dz, dz_unittest_query,
+#ifdef DZ_FULL_LENGTH_BONUS
+                                                   0,
+#endif
 			  trial == 0 ? dz_unittest_query_length
 			: trial == 1 ? 3
 			: trial == 2 ? 4
@@ -1747,10 +1781,8 @@ unittest( "extend.small" ) {
  */
 static __dz_vectorize
 int64_t dz_calc_max_rpos(
-	struct dz_s *self,
 	struct dz_forefront_s const *forefront)
 {
-	(void)self;
 	struct dz_cap_s const *pcap = forefront->mcap;
 	int32_t rpos = (pcap == dz_ccap(forefront)) ? 0 : pcap->rrem;
 	return((int64_t)rpos);									/* signed expansion */
@@ -1760,11 +1792,8 @@ int64_t dz_calc_max_rpos(
  * @fn dz_calc_max_qpos
  */
 static __dz_vectorize
-uint64_t dz_calc_max_qpos(
-	struct dz_s *self,
-	struct dz_forefront_s const *forefront)
+uint64_t dz_calc_max_qpos(struct dz_forefront_s const *forefront)
 {
-	(void)self;
 	size_t const L = sizeof(__m128i) / sizeof(uint16_t);
 	#define _dp(_cap)					( (struct dz_swgv_s const *)(_cap) - (_cap)->r.epos )
 
@@ -1772,7 +1801,7 @@ uint64_t dz_calc_max_qpos(
 	struct dz_cap_s const *pcap = forefront->mcap;
 	__m128i const maxv = _mm_set1_epi16(forefront->inc);
 	for(uint64_t p = pcap->r.spos; p < pcap->r.epos; p++) {
-		__m128i const s = _add_bonus(p, _mm_load_si128(&_dp(pcap)[p].s)); print_vector(s);
+		__m128i const s = _mm_load_si128(&_dp(pcap)[p].s); print_vector(s);
 		uint64_t eq = _mm_movemask_epi8(_mm_cmpeq_epi16(s, maxv));
 		if(eq != 0) {
 			/* tzcntq is faster but avoid using it b/c it requires relatively newer archs */
@@ -1794,14 +1823,12 @@ uint64_t dz_calc_max_qpos(
  * @brief rpos (signed int) in upper 32bit, qpos (unsigned int) in lower 32bit
  */
 static __dz_vectorize
-uint64_t dz_calc_max_pos(
-	struct dz_s *self,
-	struct dz_forefront_s const *forefront)
+uint64_t dz_calc_max_pos(struct dz_forefront_s const *forefront)
 {
 	struct dz_cap_s const *pcap = forefront->mcap;
 	if(pcap == NULL) { debug("pcap == NULL, rlen(%d)", (int32_t)forefront->rlen); return(((uint64_t)forefront->rlen)<<32); }							/* is root */
 	debug("forefront(%p), pcap(%p), rrem(%d), rlen(%d)", forefront, pcap, (int32_t)pcap->rrem, (int32_t)forefront->rlen);
-	return((dz_calc_max_rpos(self, forefront)<<32) | dz_calc_max_qpos(self, forefront));
+	return((dz_calc_max_rpos(forefront)<<32) | dz_calc_max_qpos(forefront));
 }
                                               
 #endif // DZ_INCLUDE_ONCE
@@ -1864,7 +1891,7 @@ struct dz_alignment_s *dz_trace(
     if(forefront->mcap == NULL) { debug("mcap is null"); return(NULL); }
 
 	/* detect pos */
-	uint64_t idx = dz_calc_max_qpos(self, forefront);							/* vector index, cell index */
+	uint64_t idx = dz_calc_max_qpos(forefront);							/* vector index, cell index */
 	uint64_t ref_length = 0, query_length = idx;
     
 	/* allocate aln object */
@@ -1893,13 +1920,14 @@ struct dz_alignment_s *dz_trace(
 
 	/* traceback loop */
 	#define _debug(_label) { \
-		debug("test %s, idx(%lu), rbase(%d, %c), qbase(%c), c(%d), e(%d), f(%d), s(%d), score(%d, %lu), diag(%d)", \
+		debug("test %s, idx(%lu), rbase(%d, %c), qbase(%c), c(%d), e(%d), f(%d), s(%d), score(%d, %d, %lu), diag(%d)", \
 			#_label, idx, rch, '-', '-', /*"ACGTNNNNNNNNNNNN"[rch & 0xff], "ANNNCNNNGNNNTNNN"[query->arr[idx]],*/ \
 			score, _s(e, cap, idx), _s(f, cap, idx), _s(s, pcap, idx - 1), \
 			_pair_score(self, query, rch, idx), (uint64_t)dz_pair_eq(self, query, rch, idx), \
-			(score == (_s(s, pcap, idx - 1) + _pair_score(self, query, rch, idx)))); \
+			dz_end_bonus(self, query, idx), (score == (_s(s, pcap, idx - 1) + _pair_score(self, query, rch, idx)))); \
 	}
 	#define _match(_idx) { \
+        _debug(M); \
         if(dz_inside(pcap->r.spos, _vector_idx(idx - 1), pcap->r.epos) \
            && score == (_s(s, pcap, idx - 1) + _pair_score(self, query, rch, idx))) { \
             uint64_t eq = dz_pair_eq(self, query, rch, idx); \
@@ -1930,7 +1958,12 @@ struct dz_alignment_s *dz_trace(
 	}
 
 	uint32_t rch = _load_prev_cap(s, score, _idx_asc);
-	while(1) { _debug(M); _match(_idx_asc); _ins(_idx_asc); _del(_idx_asc); dz_trap(); }
+	while(1) {
+        _match(_idx_asc);
+        _ins(_idx_asc);
+        _del(_idx_asc);
+        dz_trap();
+    }
 _trace_tail:;
 	span++;				/* adjust root span */
 
@@ -1946,7 +1979,7 @@ _trace_tail:;
 	aln->path_length = path_length;
 	aln->ref_length = ref_length;
 	aln->query_length = query_length;
-	aln->rrem = dz_calc_max_rpos(self, forefront);
+	aln->rrem = dz_calc_max_rpos(forefront);
 	aln->score = forefront->max;
 	aln->mismatch_count = cnt[0];
 	aln->match_count = cnt[1];
@@ -1986,7 +2019,11 @@ unittest( "trace" ) {
 	struct dz_s *dz = dz_init(DZ_UNITTEST_SCORE_PARAMS);
 	ut_assert(dz != NULL);
 
-	struct dz_query_s *q = dz_pack_query(dz, dz_unittest_query, dz_unittest_query_length);
+	struct dz_query_s *q = dz_pack_query(dz, dz_unittest_query,
+#ifdef DZ_FULL_LENGTH_BONUS
+                                         0,
+#endif
+                                         dz_unittest_query_length);
 	struct dz_forefront_s const *forefront = NULL;
 	struct dz_alignment_s *aln = NULL;
 
@@ -2020,6 +2057,9 @@ unittest( "trace" ) {
 #if !defined(DZ_QUAL_ADJ) && !defined(DZ_UNITTESTS_INCLUDED)
 #define DZ_UNITTESTS_INCLUDED
 #endif
+
+#undef SIMDE_ENABLE_NATIVE_ALIASES
+
 /**
  * end of dozeu.h
  */
