@@ -121,28 +121,16 @@ GBWTGraph::determine_real_nodes()
   }
 }
 
-std::vector<handle_t>
-GBWTGraph::cache_handles(const std::function<handle_t(gbwt::node_type)>& get_source_handle) const
-{
-  size_t potential_nodes = this->index->sigma() - this->index->firstNode();
-  std::vector<handle_t> handle_cache(potential_nodes / 2);
-  for(gbwt::node_type node = this->index->firstNode(); node < this->index->sigma(); node += 2)
-  {
-    handle_cache[this->node_offset(node) / 2] = get_source_handle(node);
-  }
-  return handle_cache;
-}
-
 void
-GBWTGraph::allocate_arrays(const std::function<size_t(size_t)>& get_source_length)
+GBWTGraph::allocate_arrays(const std::function<size_t(nid_t)>& get_source_length)
 {
   size_t total_length = 0;
   for(gbwt::node_type node = this->index->firstNode(); node < this->index->sigma(); node += 2)
   {
-    size_t offset = this->node_offset(node) / 2;
-    if(this->real_nodes[offset])
+    nid_t id = gbwt::Node::id(node);
+    if(this->has_node(id))
     {
-      total_length += get_source_length(offset);
+      total_length += get_source_length(id);
     }
   }
 
@@ -152,21 +140,22 @@ GBWTGraph::allocate_arrays(const std::function<size_t(size_t)>& get_source_lengt
 }
 
 void
-GBWTGraph::cache_sequences(const std::function<std::string(size_t)>& get_source_sequence)
+GBWTGraph::cache_sequences(const std::function<std::string(nid_t)>& get_source_sequence)
 {
+  size_t offset = 1; // Offset 0 is the start of the first sequence.
   for(gbwt::node_type node = this->index->firstNode(); node < this->index->sigma(); node += 2)
   {
     std::string seq;
-    size_t offset = this->node_offset(node);
-    if(this->real_nodes[offset / 2])
+    nid_t id = gbwt::Node::id(node);
+    if(this->has_node(id))
     {
-      seq = get_source_sequence(offset / 2);
+      seq = get_source_sequence(id);
     }
     this->sequences.insert(this->sequences.end(), seq.begin(), seq.end());
-    this->offsets[offset + 1] = this->sequences.size();
+    this->offsets[offset] = this->sequences.size(); offset++;
     reverse_complement_in_place(seq);
     this->sequences.insert(this->sequences.end(), seq.begin(), seq.end());
-    this->offsets[offset + 2] = this->sequences.size();
+    this->offsets[offset] = this->sequences.size(); offset++;
   }
 }
 
@@ -265,26 +254,7 @@ GBWTGraph::max_node_id() const
 bool
 GBWTGraph::follow_edges_impl(const handle_t& handle, bool go_left, const std::function<bool(const handle_t&)>& iteratee) const
 {
-  // Incoming edges correspond to the outgoing edges of the reverse node.
-  gbwt::node_type curr = handle_to_node(handle);
-  if(go_left) { curr = gbwt::Node::reverse(curr); }
-
-  // Cache the node.
-  gbwt::CachedGBWT cache(*(this->index), true);
-  gbwt::size_type cache_index = cache.findRecord(curr);
-
-  for(gbwt::rank_type outrank = 0; outrank < cache.outdegree(cache_index); outrank++)
-  {
-    gbwt::node_type next = cache.successor(cache_index, outrank);
-    if(next == gbwt::ENDMARKER) { continue; }
-
-    // If we started from the reverse node, we must reverse the successor nodes to get
-    // the predecessor nodes of the original node.
-    if(go_left) { next = gbwt::Node::reverse(next); }
-    if(!iteratee(node_to_handle(next))) { return false; }
-  }
-
-  return true;
+  return this->cached_follow_edges(this->get_single_cache(), handle, go_left, iteratee);
 }
 
 bool
@@ -320,7 +290,7 @@ GBWTGraph::get_degree(const handle_t& handle, bool go_left) const
   // Cache the node.
   gbwt::node_type curr = handle_to_node(handle);
   if(go_left) { curr = gbwt::Node::reverse(curr); }
-  gbwt::CachedGBWT cache(*(this->index), true);
+  gbwt::CachedGBWT cache = this->get_single_cache();
   gbwt::size_type cache_index = cache.findRecord(curr);
 
   // The outdegree reported by GBWT might account for the endmarker, which is
@@ -335,7 +305,7 @@ GBWTGraph::has_edge(const handle_t& left, const handle_t& right) const
 {
   // Cache the node.
   gbwt::node_type curr = handle_to_node(left);
-  gbwt::CachedGBWT cache(*(this->index), true);
+  gbwt::CachedGBWT cache = this->get_single_cache();
   gbwt::size_type cache_index = cache.findRecord(curr);
 
   for(gbwt::rank_type outrank = 0; outrank < cache.outdegree(cache_index); outrank++)
@@ -462,21 +432,6 @@ GBWTGraph::bd_find(const std::vector<handle_t>& path) const
   return result;
 }
 
-bool
-GBWTGraph::follow_paths(gbwt::SearchState state, const std::function<bool(const gbwt::SearchState&)>& iteratee) const
-{
-  gbwt::CachedGBWT cache(*(this->index), true);
-  return this->follow_paths(cache, state, iteratee);
-}
-
-bool
-GBWTGraph::follow_paths(gbwt::BidirectionalState state, bool backward,
-                        const std::function<bool(const gbwt::BidirectionalState&)>& iteratee) const
-{
-  gbwt::CachedGBWT cache(*(this->index), true);
-  return this->follow_paths(cache, state, backward, iteratee);
-}
-
 //------------------------------------------------------------------------------
 
 gbwt::SearchState
@@ -535,6 +490,31 @@ GBWTGraph::follow_paths(const gbwt::CachedGBWT& cache, gbwt::BidirectionalState
   return true;
 }
 
+bool
+GBWTGraph::cached_follow_edges(const gbwt::CachedGBWT& cache, const handle_t& handle, bool go_left,
+                               const std::function<bool(const handle_t&)>& iteratee) const
+{
+  // Incoming edges correspond to the outgoing edges of the reverse node.
+  gbwt::node_type curr = handle_to_node(handle);
+  if(go_left) { curr = gbwt::Node::reverse(curr); }
+
+  // Cache the node.
+  gbwt::size_type cache_index = cache.findRecord(curr);
+
+  for(gbwt::rank_type outrank = 0; outrank < cache.outdegree(cache_index); outrank++)
+  {
+    gbwt::node_type next = cache.successor(cache_index, outrank);
+    if(next == gbwt::ENDMARKER) { continue; }
+
+    // If we started from the reverse node, we must reverse the successor nodes to get
+    // the predecessor nodes of the original node.
+    if(go_left) { next = gbwt::Node::reverse(next); }
+    if(!iteratee(node_to_handle(next))) { return false; }
+  }
+
+  return true;
+}
+
 //------------------------------------------------------------------------------
 
 /*
