@@ -56,7 +56,7 @@ index_haplotypes(const GBWTGraph& graph, MinimizerIndex<KeyType>& index,
   // Minimizer finding.
   auto find_minimizers = [&](const std::vector<handle_t>& traversal, const std::string& seq)
   {
-    std::vector<minimizer_type> minimizers = index.minimizers(seq);
+    std::vector<minimizer_type> minimizers = index.minimizers(seq); // Calls syncmers() when appropriate.
     auto iter = traversal.begin();
     size_t node_start = 0;
     int thread_id = omp_get_thread_num();
@@ -94,7 +94,7 @@ index_haplotypes(const GBWTGraph& graph, MinimizerIndex<KeyType>& index,
     we may skip windows that cross from a reverse node to a forward node (from a forward node to a
     reverse node).
   */
-  for_each_haplotype_window(graph, index.k() + index.w() - 1, find_minimizers, (threads > 1));
+  for_each_haplotype_window(graph, index.window_bp(), find_minimizers, (threads > 1));
   for(int thread_id = 0; thread_id < threads; thread_id++) { flush_cache(thread_id); }
 }
   
