@@ -5,7 +5,6 @@
  */
 
 #include "normalize.hpp"
-#include "unchop.hpp"
 #include "simplify_siblings.hpp"
 
 #include <unordered_set>
@@ -31,7 +30,7 @@ void normalize(handlegraph::MutablePathDeletableHandleGraph* graph, int max_iter
         // for all handle graphs, or an obstacle to normality.
         
         // combine diced/chopped nodes (subpaths with no branching)
-        unchop(graph);
+        handlealgs::unchop(graph);
         // Resolve forks that shouldn't be
         simplify_siblings(graph);
         
@@ -48,7 +47,7 @@ void normalize(handlegraph::MutablePathDeletableHandleGraph* graph, int max_iter
     
     // there may now be some cut nodes that can be simplified
     // This won't change the length.
-    unchop(graph);
+    handlealgs::unchop(graph);
 }
 
 
