@@ -1,7 +1,7 @@
 #pragma once
 
 #include "../handle.hpp"
-#include "../edit.hpp"
+#include "vg/io/edit.hpp"
 #include <vg/vg.pb.h>
 #include <string>
 #include <functional>
