@@ -8,8 +8,9 @@
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
-#include "gssw.h"
 #include <vg/vg.pb.h>
+
+#include "gssw.h"
 #include "vg.hpp"
 #include "Variant.h"
 #include "Fasta.h"
@@ -17,6 +18,7 @@
 #include "utility.hpp"
 #include "statistics.hpp"
 #include "banded_global_aligner.hpp"
+#include "deletion_aligner.hpp"
 #include "dozeu_interface.hpp"
 #include "handle.hpp"
 #include "reverse_graph.hpp"
@@ -61,6 +63,11 @@ namespace vg {
     protected:
         
         GSSWAligner() = default;
+        GSSWAligner(const int8_t* _score_matrix,
+                    int8_t _gap_open,
+                    int8_t _gap_extension,
+                    int8_t _full_length_bonus,
+                    double _gc_content);
         ~GSSWAligner();
         
         // for construction
@@ -104,8 +111,12 @@ namespace vg {
                                            const vector<double>* multiplicities = nullptr) const;
         double estimate_next_best_score(int length, double min_diffs) const;
         
-        // for calling in constructors so that mapping quality can be queried
-        void init_mapping_quality(const int8_t* score_matrix, double gc_content);
+        double recover_log_base(const int8_t* score_matrix, double gc_content, double tol) const;
+        
+        bool verify_valid_log_odds_score_matrix(const int8_t* score_matrix, const double* nt_freqs) const;
+        
+        double alignment_score_partition_function(double lambda, const int8_t* score_matrix,
+                                                  const double* nt_freqs) const;
         
         // TODO: this algorithm has numerical problems, just removing it for now
         //vector<double> all_mapping_qualities_exact(vector<double> scaled_scores);
@@ -167,12 +178,21 @@ namespace vg {
         /// Qualities may be ignored by some implementations.
         virtual int32_t score_exact_match(string::const_iterator seq_begin, string::const_iterator seq_end,
                                           string::const_iterator base_qual_begin) const = 0;
-      
-        // TODO: Provide a generic way to score a mismatch.
+        /// Compute the score of a mismatch of the given range of sequence with the given qualities.
+        /// Qualities may be ignored by some implementations.
+        /// Note that the return value is SIGNED, and almost certainly NEGATIVE, because mismatches are bad.
+        virtual int32_t score_mismatch(string::const_iterator seq_begin, string::const_iterator seq_end,
+                                       string::const_iterator base_qual_begin) const = 0;
         
+        virtual int32_t score_full_length_bonus(bool left_side, string::const_iterator seq_begin,
+                                                string::const_iterator seq_end,
+                                                string::const_iterator base_qual_begin) const = 0;
+        
+        virtual int32_t score_full_length_bonus(bool left_side, const Alignment& alignment) const = 0;
+                
         /// Compute the score of a path against the given range of subsequence with the given qualities.
         virtual int32_t score_partial_alignment(const Alignment& alignment, const HandleGraph& graph, const Path& path,
-                                                string::const_iterator seq_begin) const = 0;
+                                                string::const_iterator seq_begin, bool no_read_end_scoring = false) const = 0;
         
         /// Returns the score of an insert or deletion of the given length
         int32_t score_gap(size_t gap_length) const;
@@ -234,6 +254,9 @@ namespace vg {
         /// The longest gap detectable from any read position without soft-clipping
         size_t longest_detectable_gap(const Alignment& alignment) const;
         
+        /// The longest gap detectable from any read position without soft-clipping, for a generic read.
+        size_t longest_detectable_gap(size_t read_length) const;
+        
         /// Use the score values in the aligner to score the given alignment,
         /// scoring gaps caused by jumping between between nodes using a custom
         /// gap length estimation function (which takes the from position, the
@@ -241,14 +264,14 @@ namespace vg {
         /// length).
         ///
         /// May include full length bonus or not. TODO: bool flags are bad.
-        virtual int32_t score_gappy_alignment(const Alignment& aln,
+        virtual int32_t score_discontiguous_alignment(const Alignment& aln,
             const function<size_t(pos_t, pos_t, size_t)>& estimate_distance,
             bool strip_bonuses = false) const;
         
         /// Use the score values in the aligner to score the given alignment assuming
         /// that there are no gaps between Mappings in the Path
-        virtual int32_t score_ungapped_alignment(const Alignment& aln,
-                                                 bool strip_bonuses = false) const;
+        virtual int32_t score_contiguous_alignment(const Alignment& aln,
+                                                   bool strip_bonuses = false) const;
 
         /// Without necessarily rescoring the entire alignment, return the score
         /// of the given alignment with bonuses removed. Assumes that bonuses
@@ -257,6 +280,7 @@ namespace vg {
         virtual int32_t remove_bonuses(const Alignment& aln, bool pinned = false, bool pin_left = false) const;
         
         // members
+        DeletionAligner deletion_aligner;
         int8_t* nt_table = nullptr;
         int8_t* score_matrix = nullptr;
         int8_t match;
@@ -342,12 +366,21 @@ namespace vg {
                                   string::const_iterator base_qual_begin) const;
         int32_t score_exact_match(const string& sequence) const;
         int32_t score_exact_match(string::const_iterator seq_begin, string::const_iterator seq_end) const;
+        int32_t score_mismatch(string::const_iterator seq_begin, string::const_iterator seq_end,
+                               string::const_iterator base_qual_begin) const;
         
         /// Score a mismatch given just the length. Only possible since we ignore qualities.
+        /// Return value is SIGNED, and almost certainly NEGATIVE
         int32_t score_mismatch(size_t length) const;
+        
+        int32_t score_full_length_bonus(bool left_side, string::const_iterator seq_begin,
+                                        string::const_iterator seq_end,
+                                        string::const_iterator base_qual_begin) const;
+        
+        int32_t score_full_length_bonus(bool left_side, const Alignment& alignment) const;
 
         int32_t score_partial_alignment(const Alignment& alignment, const HandleGraph& graph, const Path& path,
-                                        string::const_iterator seq_begin) const;
+                                        string::const_iterator seq_begin, bool no_read_end_scoring = false) const;
         
     private:
         
@@ -372,7 +405,7 @@ namespace vg {
                        int8_t _full_length_bonus = default_full_length_bonus,
                        double _gc_content = default_gc_content);
 
-        ~QualAdjAligner(void) = default;
+        ~QualAdjAligner(void);
         
         // base quality adjusted counterparts to functions of same name from Aligner
         
@@ -396,18 +429,31 @@ namespace vg {
         int32_t score_exact_match(const string& sequence, const string& base_quality) const;
         int32_t score_exact_match(string::const_iterator seq_begin, string::const_iterator seq_end,
                                   string::const_iterator base_qual_begin) const;
-                                  
+        int32_t score_mismatch(string::const_iterator seq_begin, string::const_iterator seq_end,
+                               string::const_iterator base_qual_begin) const;
+        
+        int32_t score_full_length_bonus(bool left_side, string::const_iterator seq_begin,
+                                        string::const_iterator seq_end,
+                                        string::const_iterator base_qual_begin) const;
+        
+        int32_t score_full_length_bonus(bool left_side, const Alignment& alignment) const;
+        
         int32_t score_partial_alignment(const Alignment& alignment, const HandleGraph& graph, const Path& path,
-                                        string::const_iterator seq_begin) const;
+                                        string::const_iterator seq_begin, bool no_read_end_scoring = false) const;
         
         
-    private:
+    protected:
+        
+        int8_t* qual_adjusted_matrix(const int8_t* score_matrix, double gc_content, uint32_t max_qual) const;
 
+        int8_t* qual_adjusted_bonuses(int8_t _full_length_bonus, uint32_t max_qual) const;
+        
         // internal function interacting with gssw for pinned and local alignment
         void align_internal(Alignment& alignment, vector<Alignment>* multi_alignments, const HandleGraph& g,
                             bool pinned, bool pin_left, int32_t max_alt_alns,
                             bool traceback_aln) const;
         
+        int8_t* qual_adj_full_length_bonuses = nullptr;
 
         // members
         vector<QualAdjXdropAligner> xdrops;
