@@ -1,7 +1,9 @@
 #include "alignment.hpp"
+#include "vg/io/gafkluge.hpp"
 
 #include <sstream>
-#include <regex>
+
+using namespace vg::io;
 
 namespace vg {
 
@@ -91,11 +93,22 @@ bam_hdr_t* hts_file_header(string& filename, string& header) {
 }
 
 bam_hdr_t* hts_string_header(string& header,
-                             map<string, int64_t>& path_length,
-                             map<string, string>& rg_sample) {
+                             const map<string, int64_t>& path_length,
+                             const map<string, string>& rg_sample) {
+    
+    // Copy the map into a vecotr in its own order
+    vector<pair<string, int64_t>> path_order_and_length(path_length.begin(), path_length.end());
+    
+    // Make header in that order.
+    return hts_string_header(header, path_order_and_length, rg_sample);
+}
+
+bam_hdr_t* hts_string_header(string& header,
+                             const vector<pair<string, int64_t>>& path_order_and_length,
+                             const map<string, string>& rg_sample) {
     stringstream hdr;
     hdr << "@HD\tVN:1.5\tSO:unknown\n";
-    for (auto& p : path_length) {
+    for (auto& p : path_order_and_length) {
         hdr << "@SQ\tSN:" << p.first << "\t" << "LN:" << p.second << "\n";
     }
     for (auto& s : rg_sample) {
@@ -125,7 +138,7 @@ bool get_next_alignment_from_fastq(gzFile fp, char* buffer, size_t len, Alignmen
         } else {
             throw runtime_error("Found unexpected delimiter " + name.substr(0,1) + " in fastq/fasta input");
         }
-        name = name.substr(1, name.find(' ')); // trim off leading @ and things after the first whitespace
+        name = name.substr(1, name.find(' ') - 1); // trim off leading @ and things after the first whitespace
         // keep trailing /1 /2
         alignment.set_name(name);
     } else { return false; }
@@ -165,186 +178,6 @@ bool get_next_alignment_pair_from_fastqs(gzFile fp1, gzFile fp2, char* buffer, s
     return get_next_alignment_from_fastq(fp1, buffer, len, mate1) && get_next_alignment_from_fastq(fp2, buffer, len, mate2);
 }
 
-size_t unpaired_for_each_parallel(function<bool(Alignment&)> get_read_if_available, function<void(Alignment&)> lambda) {
-
-    size_t nLines = 0;
-    vector<Alignment> *batch = nullptr;
-    // number of batches currently being processed
-    uint64_t batches_outstanding = 0;
-#pragma omp parallel default(none) shared(batches_outstanding, batch, nLines, get_read_if_available, lambda)
-#pragma omp single
-    {
-        
-        // number of reads in each batch
-        const uint64_t batch_size = 1 << 9; // 512
-        // max # of such batches to be holding in memory
-        uint64_t max_batches_outstanding = 1 << 9; // 512
-        // max # we will ever increase the batch buffer to
-        const uint64_t max_max_batches_outstanding = 1 << 13; // 8192
-        
-        // alignments to hold the incoming data
-        Alignment aln;
-        // did we find the end of the file yet?
-        bool more_data = true;
-        
-        while (more_data) {
-            // init a new batch
-            batch = new std::vector<Alignment>();
-            batch->reserve(batch_size);
-            
-            // load up to the batch-size number of reads
-            for (int i = 0; i < batch_size; i++) {
-                
-                more_data = get_read_if_available(aln);
-                
-                if (more_data) {
-                    batch->emplace_back(std::move(aln));
-                    nLines++;
-                }
-                else {
-                    break;
-                }
-            }
-            
-            // did we get a batch?
-            if (batch->size()) {
-                
-                // how many batch tasks are outstanding currently, including this one?
-                uint64_t current_batches_outstanding;
-#pragma omp atomic capture
-                current_batches_outstanding = ++batches_outstanding;
-                
-                if (current_batches_outstanding >= max_batches_outstanding) {
-                    // do this batch in the current thread because we've spawned the maximum number of
-                    // concurrent batch tasks
-                    for (auto& aln : *batch) {
-                        lambda(aln);
-                    }
-                    delete batch;
-#pragma omp atomic capture
-                    current_batches_outstanding = --batches_outstanding;
-                    
-                    if (4 * current_batches_outstanding / 3 < max_batches_outstanding
-                        && max_batches_outstanding < max_max_batches_outstanding) {
-                        // we went through at least 1/4 of the batch buffer while we were doing this thread's batch
-                        // this looks risky, since we want the batch buffer to stay populated the entire time we're
-                        // occupying this thread on compute, so let's increase the batch buffer size
-                        
-                        max_batches_outstanding *= 2;
-                    }
-                }
-                else {
-                    // spawn a new task to take care of this batch
-#pragma omp task default(none) firstprivate(batch) shared(batches_outstanding, lambda)
-                    {
-                        for (auto& aln : *batch) {
-                            lambda(aln);
-                        }
-                        delete batch;
-#pragma omp atomic update
-                        batches_outstanding--;
-                    }
-                }
-            }
-        }
-    }
-    return nLines;
-}
-
-size_t paired_for_each_parallel_after_wait(function<bool(Alignment&, Alignment&)> get_pair_if_available,
-                                           function<void(Alignment&, Alignment&)> lambda,
-                                           function<bool(void)> single_threaded_until_true) {
-    
-    
-    size_t nLines = 0;
-    vector<pair<Alignment, Alignment> > *batch = nullptr;
-    // number of batches currently being processed
-    uint64_t batches_outstanding = 0;
-    
-#pragma omp parallel default(none) shared(batches_outstanding, batch, nLines, get_pair_if_available, single_threaded_until_true, lambda)
-#pragma omp single
-    {
-        
-        // number of pairs in each batch
-        const uint64_t batch_size = 1 << 9; // 512
-        // max # of such batches to be holding in memory
-        uint64_t max_batches_outstanding = 1 << 9; // 512
-        // max # we will ever increase the batch buffer to
-        const uint64_t max_max_batches_outstanding = 1 << 13; // 8192
-        
-        // alignments to hold the incoming data
-        Alignment mate1, mate2;
-        // did we find the end of the file yet?
-        bool more_data = true;
-        
-        while (more_data) {
-            // init a new batch
-            batch = new std::vector<pair<Alignment, Alignment>>();
-            batch->reserve(batch_size);
-            
-            // load up to the batch-size number of pairs
-            for (int i = 0; i < batch_size; i++) {
-                
-                more_data = get_pair_if_available(mate1, mate2);
-                
-                if (more_data) {
-                    batch->emplace_back(std::move(mate1), std::move(mate2));
-                    nLines++;
-                }
-                else {
-                    break;
-                }
-            }
-            
-            // did we get a batch?
-            if (batch->size()) {
-                // how many batch tasks are outstanding currently, including this one?
-                uint64_t current_batches_outstanding;
-#pragma omp atomic capture
-                current_batches_outstanding = ++batches_outstanding;
-                
-                bool do_single_threaded = !single_threaded_until_true();
-                if (current_batches_outstanding >= max_batches_outstanding || do_single_threaded) {
-                    // do this batch in the current thread because we've spawned the maximum number of
-                    // concurrent batch tasks or because we are directed to work in a single thread
-                    for (auto& p : *batch) {
-                        lambda(p.first, p.second);
-                    }
-                    delete batch;
-#pragma omp atomic capture
-                    current_batches_outstanding = --batches_outstanding;
-                    
-                    if (4 * current_batches_outstanding / 3 < max_batches_outstanding
-                        && max_batches_outstanding < max_max_batches_outstanding
-                        && !do_single_threaded) {
-                        // we went through at least 1/4 of the batch buffer while we were doing this thread's batch
-                        // this looks risky, since we want the batch buffer to stay populated the entire time we're
-                        // occupying this thread on compute, so let's increase the batch buffer size
-                        // (skip this adjustment if you're in single-threaded mode and thus expect the buffer to be
-                        // empty)
-                        
-                        max_batches_outstanding *= 2;
-                    }
-                }
-                else {
-                    // spawn a new task to take care of this batch
-#pragma omp task default(none) firstprivate(batch) shared(batches_outstanding, lambda)
-                    {
-                        for (auto& p : *batch) {
-                            lambda(p.first, p.second);
-                        }
-                        delete batch;
-#pragma omp atomic update
-                        batches_outstanding--;
-                    }
-                }
-            }
-        }
-    }
-    
-    return nLines;
-}
-
 size_t fastq_unpaired_for_each_parallel(const string& filename, function<void(Alignment&)> lambda) {
     
     gzFile fp = (filename != "-") ? gzopen(filename.c_str(), "r") : gzdopen(fileno(stdin), "r");
@@ -463,6 +296,7 @@ size_t fastq_paired_interleaved_for_each(const string& filename, function<void(A
     return nLines;
 }
 
+
 size_t fastq_paired_two_files_for_each(const string& file1, const string& file2, function<void(Alignment&, Alignment&)> lambda) {
     gzFile fp1 = (file1 != "-") ? gzopen(file1.c_str(), "r") : gzdopen(fileno(stdin), "r");
     if (!fp1) {
@@ -546,38 +380,6 @@ void parse_rg_sample_map(char* hts_header, map<string, string>& rg_sample) {
     }
 }
 
-short quality_char_to_short(char c) {
-    return static_cast<short>(c) - 33;
-}
-
-char quality_short_to_char(short i) {
-    return static_cast<char>(i + 33);
-}
-
-void alignment_quality_short_to_char(Alignment& alignment) {
-    alignment.set_quality(string_quality_short_to_char(alignment.quality()));
-}
-
-string string_quality_short_to_char(const string& quality) {
-    string buffer; buffer.resize(quality.size());
-    for (int i = 0; i < quality.size(); ++i) {
-        buffer[i] = quality_short_to_char(quality[i]);
-    }
-    return buffer;
-}
-
-void alignment_quality_char_to_short(Alignment& alignment) {
-    alignment.set_quality(string_quality_char_to_short(alignment.quality()));
-}
-
-string string_quality_char_to_short(const string& quality) {
-    string buffer; buffer.resize(quality.size());
-    for (int i = 0; i < quality.size(); ++i) {
-        buffer[i] = quality_char_to_short(quality[i]);
-    }
-    return buffer;
-}
-
 // Internal conversion function for both paired and unpaired codepaths
 string alignment_to_sam_internal(const Alignment& alignment,
                                  const string& refseq,
@@ -741,7 +543,7 @@ bam1_t* alignment_to_bam_internal(bam_hdr_t* header,
                                   bool paired,
                                   const int32_t tlen_max) {
     
-    // this table does seem to be reproduced in htslib publicly, so I'm copying
+    // this table doesn't seem to be reproduced in htslib publicly, so I'm copying
     // it from the CRAM conversion code
     static const char nt_encoding[256] = {
         15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
@@ -766,13 +568,14 @@ bam1_t* alignment_to_bam_internal(bam_hdr_t* header,
     bam1_t* bam = bam_init1();
     
     // strip the pair order identifiers
-    string alignment_name;
-    if (paired) {
+    string alignment_name = alignment.name();
+    if (paired && alignment_name.size() >= 2) {
         // We need to strip the /1 and /2 or _1 and _2 from paired reads so the two ends have the same name.
-        alignment_name = regex_replace(alignment.name(), regex("[/_][12]$"), "");
-    } else {
-        // Keep the alignment name as is because even if the name looks paired, the reads are semantically unpaired.
-        alignment_name = alignment.name();
+        char c1 = alignment_name[alignment_name.size() - 2];
+        char c2 = alignment_name[alignment_name.size() - 1];
+        if ((c1 == '_' || c1 == '/') && (c2 == '1' || c2 == '2')) {
+            alignment_name = alignment_name.substr(0, alignment_name.size() - 2);
+        }
     }
     
     // calculate the size in bytes of the variable length fields (which are all concatenated in memory)
@@ -890,9 +693,14 @@ bam1_t* alignment_to_bam_internal(bam_hdr_t* header,
     uint8_t* seq_data = (uint8_t*) (cigar_data + cigar.size());
     const string* seq = &alignment.sequence();
     string rev_seq;
+    const string* qual = &alignment.quality();
+    string rev_qual;
     if (refrev) {
+        // Sequence and quality both need to be flipped to target forward orientation
         rev_seq = reverse_complement(*seq);
         seq = &rev_seq;
+        reverse_copy(qual->begin(), qual->end(), back_inserter(rev_qual));
+        qual = &rev_qual;
     }
     for (size_t i = 0; i < alignment.sequence().size(); i += 2) {
         if (i + 1 < alignment.sequence().size()) {
@@ -911,7 +719,7 @@ bam1_t* alignment_to_bam_internal(bam_hdr_t* header,
             qual_data[i] = '\xff';
         }
         else {
-            qual_data[i] = alignment.quality().at(i);
+            qual_data[i] = qual->at(i);
         }
     }
     
@@ -1007,15 +815,6 @@ string mapping_string(const string& source, const Mapping& mapping) {
     return result;
 }
 
-inline void append_cigar_operation(const int length, const char operation, vector<pair<int, char>>& cigar) {
-    if (cigar.empty() || operation != cigar.back().second) {
-        cigar.emplace_back(length, operation);
-    }
-    else {
-        cigar.back().first += length;
-    }
-}
-
 void mapping_cigar(const Mapping& mapping, vector<pair<int, char>>& cigar) {
     for (const auto& edit : mapping.edit()) {
         if (edit.from_length() && edit.from_length() == edit.to_length()) {
@@ -1117,6 +916,12 @@ vector<pair<int, char>> cigar_against_path(const Alignment& alignment, bool on_r
 
     // handle soft clips, which are just insertions at the start or end
     // back
+    if (cigar.size() > 1 && cigar.back().second == 'D' && cigar[cigar.size() - 2].second == 'I') {
+        // Swap insert to the outside so it can be a softclip.
+        // When making the CIGAR we should put D before I but when flipping the
+        // strand they may switch.
+        std::swap(cigar.back(), cigar[cigar.size() - 2]);
+    }
     if (cigar.back().second == 'I') {
         // make sure we stay in the reference sequence when suppressing the softclips
         if (cigar.back().first <= softclip_suppress
@@ -1127,6 +932,10 @@ vector<pair<int, char>> cigar_against_path(const Alignment& alignment, bool on_r
         }
     }
     // front
+    if (cigar.size() > 1 && cigar.front().second == 'D' && cigar[1].second == 'I') {
+        // Swap insert to the outside so it can be a softclip
+        std::swap(cigar.front(), cigar[1]);
+    }
     if (cigar.front().second == 'I') {
         // make sure we stay in the reference sequence when suppressing the softclips
         if (cigar.front().first <= softclip_suppress
@@ -1141,6 +950,41 @@ vector<pair<int, char>> cigar_against_path(const Alignment& alignment, bool on_r
     return cigar;
 }
 
+void consolidate_ID_runs(vector<pair<int, char>>& cigar) {
+    
+    size_t removed = 0;
+    for (size_t i = 0, j = 0; i < cigar.size(); ++j) {
+        if (j == cigar.size() || (cigar[j].second != 'I' && cigar[j].second != 'D')) {
+            // this is the end boundary of a runs of I/D operations
+            if (j - i >= 3) {
+                // we have at least 3 adjacent I/D operations, which means they should
+                // be re-consolidated
+                int d_total = 0, i_total = 0;
+                for (size_t k = i - removed, end = j - removed; k < end; ++k) {
+                    if (cigar[k].second == 'D') {
+                        d_total += cigar[k].first;
+                    }
+                    else {
+                        i_total += cigar[k].first;
+                    }
+                }
+                
+                cigar[i - removed] = make_pair(d_total, 'D');
+                cigar[i - removed + 1] = make_pair(i_total, 'I');
+                
+                // mark that we've
+                removed += j - i - 2;
+            }
+            // move the start of the next I/D run beyond the current operation
+            i = j + 1;
+        }
+        if (j < cigar.size()) {
+            cigar[j - removed] = cigar[j];
+        }
+    }
+    cigar.resize(cigar.size() - removed);
+}
+
 pair<int32_t, int32_t> compute_template_lengths(const int64_t& pos1, const vector<pair<int, char>>& cigar1,
     const int64_t& pos2, const vector<pair<int, char>>& cigar2) {
 
@@ -1164,7 +1008,7 @@ pair<int32_t, int32_t> compute_template_lengths(const int64_t& pos1, const vecto
                 // Bases are matched. Count them in the bounds and execute the operation
                 low = min(low, here);
                 here += item.first;
-                high = max(high, here - 1);
+                high = max(high, here);
             } else if (item.second == 'D') {
                 // Only other way to advance in the reference
                 here += item.first;
@@ -1229,8 +1073,6 @@ int32_t sam_flag(const Alignment& alignment, bool on_reverse_strand, bool paired
         flag |= BAM_FSECONDARY;
     }
     
-    
-    
     return flag;
 }
 
@@ -1832,6 +1674,56 @@ void normalize_alignment(Alignment& alignment) {
     }
 }
 
+bool uses_Us(const Alignment& alignment) {
+    
+    for (char nt : alignment.sequence()) {
+        switch (nt) {
+            case 'U':
+                return true;
+                break;
+                
+            case 'T':
+                return false;
+                break;
+                
+            default:
+                break;
+        }
+    }
+    return false;
+}
+
+void convert_alignment_char(Alignment& alignment, char from, char to) {
+    auto& seq = *alignment.mutable_sequence();
+    for (size_t i = 0; i < seq.size(); ++i) {
+        if (seq[i] == from) {
+            seq[i] = to;
+        }
+    }
+    if (alignment.has_path()) {
+        for (Mapping& mapping : *alignment.mutable_path()->mutable_mapping()) {
+            for (Edit& edit : *mapping.mutable_edit()) {
+                if (!edit.sequence().empty()) {
+                    auto& eseq = *edit.mutable_sequence();
+                    for (size_t i = 0; i < eseq.size(); ++i) {
+                        if (eseq[i] == from) {
+                            eseq[i] = to;
+                        }
+                    }
+                }
+            }
+        }
+    }
+}
+
+void convert_Us_to_Ts(Alignment& alignment) {
+    convert_alignment_char(alignment, 'U', 'T');
+}
+
+void convert_Ts_to_Us(Alignment& alignment) {
+    convert_alignment_char(alignment, 'T', 'U');
+}
+
 map<id_t, int> alignment_quality_per_node(const Alignment& aln) {
     map<id_t, int> quals;
     int to_pos = 0; // offset in quals
@@ -1917,7 +1809,7 @@ void parse_bed_regions(istream& bedstream,
 
         if (ss.fail()) {
             // Skip lines that can't be parsed
-            cerr << "Error parsing bed line " << line << ": " << row << endl;
+            cerr << "warning: Error parsing bed line " << line << ", skipping: " << row << endl;
             continue;
         } 
         
@@ -1929,6 +1821,20 @@ void parse_bed_regions(istream& bedstream,
             continue;
         }
         
+        if (ebuf > graph->get_path_length(path_handle)) {
+            // Skip ends that are too late
+            cerr << "warning: out of range path end " << ebuf << " > " << graph->get_path_length(path_handle)
+                << " in bed line " << line << ", skipping: " << row << endl;
+            continue;
+        }
+        
+        if (sbuf >= graph->get_path_length(path_handle)) {
+            // Skip starts that are too late
+            cerr << "warning: out of range path start " << sbuf << " >= " << graph->get_path_length(path_handle)
+                << " in bed line " << line << ", skipping: " << row << endl;
+            continue;
+        }
+        
         // Try parsing the optional fields. If they fail, ignore the problem, because they're optional.
         ss >> name;
         ss >> score;
@@ -1977,8 +1883,10 @@ void parse_gff_regions(istream& gffstream,
         getline(ss, source, '\t');
         getline(ss, type, '\t');
         getline(ss, buf, '\t');
-        sbuf = atoi(buf.c_str());
+        // Convert to 0-based 
+        sbuf = atoi(buf.c_str()) - 1;
         getline(ss, buf, '\t');
+        // 1-based inclusive == 0-based exclusive
         ebuf = atoi(buf.c_str());
 
         if (ss.fail() || !(sbuf < ebuf)) {
