@@ -8,20 +8,18 @@
 #include "path.hpp"
 #include "position.hpp"
 #include <vg/vg.pb.h>
-#include "edit.hpp"
-#include "htslib/hfile.h"
-#include "htslib/hts.h"
-#include "htslib/sam.h"
-#include "htslib/vcf.h"
+#include "vg/io/edit.hpp"
+#include <htslib/hfile.h>
+#include <htslib/hts.h>
+#include <htslib/sam.h>
+#include <htslib/vcf.h>
 #include "handle.hpp"
-#include "gafkluge.hpp"
+#include "vg/io/alignment_io.hpp"
 
 namespace vg {
 
 const char* const BAM_DNA_LOOKUP = "=ACMGRSVTWYHKDBN";
 
-const uint64_t DEFAULT_PARALLEL_BATCHSIZE = 512;
-
 int hts_for_each(string& filename, function<void(Alignment&)> lambda);
 int hts_for_each_parallel(string& filename, function<void(Alignment&)> lambda);
 int hts_for_each(string& filename, function<void(Alignment&)> lambda,
@@ -56,33 +54,13 @@ size_t fastq_paired_two_files_for_each_parallel_after_wait(const string& file1,
                                                            function<void(Alignment&, Alignment&)> lambda,
                                                            function<bool(void)> single_threaded_until_true);
 
-// single gaf
-bool get_next_alignment_from_gaf(const HandleGraph& graph, htsFile* fp, kstring_t& s_buffer, gafkluge::GafRecord& g_buffer,
-                                 Alignment& alignment);
-bool get_next_interleaved_alignment_pair_from_gaf(const HandleGraph& graph, htsFile* fp, kstring_t& s_buffer,
-                                                  gafkluge::GafRecord& g_buffer, Alignment& mate1, Alignment& mate2);
-size_t gaf_unpaired_for_each(const HandleGraph& graph, const string& filename, function<void(Alignment&)> lambda);
-size_t gaf_paired_interleaved_for_each(const HandleGraph& graph, const string& filename,
-                                       function<void(Alignment&, Alignment&)> lambda);
-// parallel gaf
-size_t gaf_unpaired_for_each_parallel(const HandleGraph& graph, const string& filename,
-                                      function<void(Alignment&)> lambda,
-                                      uint64_t batch_size = DEFAULT_PARALLEL_BATCHSIZE);
-size_t gaf_paired_interleaved_for_each_parallel(const HandleGraph& graph, const string& filename,
-                                                function<void(Alignment&, Alignment&)> lambda,
-                                                uint64_t batch_size = DEFAULT_PARALLEL_BATCHSIZE);
-size_t gaf_paired_interleaved_for_each_parallel_after_wait(const HandleGraph& graph, const string& filename,
-                                                           function<void(Alignment&, Alignment&)> lambda,
-                                                           function<bool(void)> single_threaded_until_true,
-                                                           uint64_t batch_size = DEFAULT_PARALLEL_BATCHSIZE);
-// gaf conversion
-gafkluge::GafRecord alignment_to_gaf(const HandleGraph& graph, const Alignment& aln, bool cs_cigar = true, bool base_quals = true);
-void gaf_to_alignment(const HandleGraph& graph, const gafkluge::GafRecord& gaf, Alignment& aln);
-
 bam_hdr_t* hts_file_header(string& filename, string& header);
 bam_hdr_t* hts_string_header(string& header,
-                             map<string, int64_t>& path_length,
-                             map<string, string>& rg_sample);
+                             const map<string, int64_t>& path_length,
+                             const map<string, string>& rg_sample);
+bam_hdr_t* hts_string_header(string& header,
+                             const vector<pair<string, int64_t>>& path_order_and_length,
+                             const map<string, string>& rg_sample);
 void write_alignment_to_file(const Alignment& aln, const string& filename);
 
 void mapping_cigar(const Mapping& mapping, vector<pair<int, char> >& cigar);
@@ -95,6 +73,38 @@ Alignment bam_to_alignment(const bam1_t *b, map<string, string>& rg_sample, cons
                            const PathPositionHandleGraph* graph);
 Alignment bam_to_alignment(const bam1_t *b, map<string, string>& rg_sample);
 
+/**
+ * Add a CIGAR operation to a vector representing the parsed CIGAR string.
+ *
+ * Coalesces adjacent operations of the same type. Coalesces runs of inserts
+ * and deletes into a signle delete followed by a single insert.
+ */
+inline void append_cigar_operation(const int length, const char operation, vector<pair<int, char>>& cigar) {
+    if (cigar.empty()) {
+        // Always append to an empty CIGAR
+        cigar.emplace_back(length, operation);
+    } else if (operation != cigar.back().second) {
+        // We have changed operations
+        if (operation == 'D' && cigar.back().second == 'I') {
+            // This deletion needs to come before the adjacent insertion
+            if (cigar.size() > 1 && cigar[cigar.size() - 2].second == 'D') {
+                // Add to the deletion that laready exists before the insertion
+                cigar[cigar.size() - 2].first += length;
+            } else {
+                // Create a new deletion
+                cigar.emplace_back(length, operation);
+                // Put it under the insertion
+                std::swap(cigar[cigar.size() - 2], cigar.back());
+            }
+        } else {
+            // This is an ordinary change of operations.
+            cigar.emplace_back(length, operation);
+        }
+    } else {
+        cigar.back().first += length;
+    }
+}
+
 /**
  * Convert a paired Alignment to a BAM record. If the alignment is unmapped,
  * refpos must be -1. Otherwise, refpos must be the position on the reference
@@ -183,6 +193,9 @@ int32_t determine_flag(const Alignment& alignment,
 /// which is why it is passed by reference.
 vector<pair<int, char>> cigar_against_path(const Alignment& alignment, bool on_reverse_strand, int64_t& pos, size_t path_len, size_t softclip_suppress);
 
+/// Merge runs of successive I/D operations into a single I and D
+void consolidate_ID_runs(vector<pair<int, char>>& cigar);
+
 void mapping_against_path(Alignment& alignment, const bam1_t *b,
                           const PathPositionHandleGraph* graph, bool on_reverse_strand);
 
@@ -193,12 +206,6 @@ pair<int32_t, int32_t> compute_template_lengths(const int64_t& pos1, const vecto
     const int64_t& pos2, const vector<pair<int, char>>& cigar2);
 
 int32_t sam_flag(const Alignment& alignment, bool on_reverse_strand, bool paired);
-short quality_char_to_short(char c);
-char quality_short_to_char(short i);
-string string_quality_char_to_short(const string& quality);
-string string_quality_short_to_char(const string& quality);
-void alignment_quality_char_to_short(Alignment& alignment);
-void alignment_quality_short_to_char(Alignment& alignment);
 void parse_rg_sample_map(char* hts_header, map<string, string>& rg_sample);
 int alignment_to_length(const Alignment& a);
 int alignment_from_length(const Alignment& a);
@@ -296,6 +303,7 @@ Alignment target_alignment(const PathPositionHandleGraph* graph, const string& n
 /// The edits are inserted into the generated Alignment, cut as necessary to fit into the Alignment's Mappings.
 Alignment target_alignment(const PathPositionHandleGraph* graph, const string& name, size_t pos1, size_t pos2,
                            const string& feature, bool is_reverse, Mapping& cigar_mapping);
+
 }
 
 #endif
