@@ -3,7 +3,7 @@
 
 #include <vg/vg.pb.h>
 #include <iostream>
-#include "json2pb.h"
+#include "vg/io/json2pb.h"
 #include "handle.hpp"
 #include "utility.hpp"
 #include "gcsa/gcsa.h"
