@@ -145,11 +145,13 @@ Path GaplessExtension::to_path(const HandleGraph& graph, const std::string& sequ
 GaplessExtender::GaplessExtender() :
     graph(nullptr), aligner(nullptr)
 {
+    this->init_mask();
 }
 
 GaplessExtender::GaplessExtender(const gbwtgraph::GBWTGraph& graph, const Aligner& aligner) :
     graph(&graph), aligner(&aligner)
 {
+    this->init_mask();
 }
 
 //------------------------------------------------------------------------------
@@ -267,8 +269,8 @@ void match_backward(GaplessExtension& match, const std::string& seq, gbwtgraph::
 }
 
 // Sort full-length extensions by internal_score, remove ones that are not
-// full-length alignments, remove duplicates, and return the best two
-// extensions that have sufficiently low overlap.
+// full-length alignments, remove duplicates, and return the best extensions
+// that have sufficiently low overlap.
 void handle_full_length(const HandleGraph& graph, std::vector<GaplessExtension>& result, double overlap_threshold) {
     std::sort(result.begin(), result.end(), [](const GaplessExtension& a, const GaplessExtension& b) -> bool {
         if (a.full() && b.full()) {
@@ -281,20 +283,20 @@ void handle_full_length(const HandleGraph& graph, std::vector<GaplessExtension>&
         if (!(result[i].full())) {
             break; // No remaining full-length extensions.
         }
-        if (tail == 0) {
-            tail = 1; // Best extension.
-        } else {
-            if (result[i] == result.front()) {
-                continue; // Duplicate.
-            }
-            if (result[i].overlap(graph, result.front()) < overlap_threshold * result.front().length()) {
-                if (i > tail) {
-                    result[tail] = std::move(result[i]);
-                }
-                tail++;
-                break; // Found a non-overlapping extension.
+        bool overlap = false;
+        for (size_t prev = 0; prev < tail; prev++) {
+            if (result[i].overlap(graph, result[prev]) > overlap_threshold * result[prev].length()) {
+                overlap = true;
+                break;
             }
         }
+        if (overlap) {
+            continue;
+        }
+        if (i > tail) {
+            result[tail] = std::move(result[i]);
+        }
+        tail++;
     }
     result.resize(tail);
 }
@@ -311,7 +313,13 @@ void remove_duplicates(std::vector<GaplessExtension>& result) {
         if (a.state.forward.node != b.state.forward.node) {
             return (a.state.forward.node < b.state.forward.node);
         }
-        return (a.state.backward.range < b.state.backward.range);
+        if (a.state.backward.range != b.state.backward.range) {
+           return (a.state.backward.range < b.state.backward.range);
+        }
+        if (a.state.forward.range != b.state.forward.range) {
+           return (a.state.forward.range < b.state.forward.range);
+        }
+        return (a.offset < b.offset);
     };
     std::sort(result.begin(), result.end(), sort_order);
     size_t tail = 0;
@@ -495,13 +503,14 @@ bool trim_mismatches(GaplessExtension& extension, const gbwtgraph::CachedGBWTGra
 
 //------------------------------------------------------------------------------
 
-std::vector<GaplessExtension> GaplessExtender::extend(cluster_type& cluster, const std::string& sequence, const gbwtgraph::CachedGBWTGraph* cache, size_t max_mismatches, double overlap_threshold) const {
+std::vector<GaplessExtension> GaplessExtender::extend(cluster_type& cluster, std::string sequence, const gbwtgraph::CachedGBWTGraph* cache, size_t max_mismatches, double overlap_threshold) const {
 
     std::vector<GaplessExtension> result;
     if (this->graph == nullptr || this->aligner == nullptr || cluster.empty() || sequence.empty()) {
         return result;
     }
     result.reserve(cluster.size());
+    this->mask_sequence(sequence);
 
     // Allocate a cache if we were not provided with one.
     bool free_cache = (cache == nullptr);
@@ -559,7 +568,7 @@ std::vector<GaplessExtension> GaplessExtender::extend(cluster_type& cluster, con
 
             // Case 1: Extend to the right.
             if (!curr.right_maximal) {
-                bool found_extension = false;
+                size_t num_extensions = 0;
                 // Always allow at least max_mismatches / 2 mismatches in the current flank.
                 uint32_t mismatch_limit = std::max(
                     static_cast<uint32_t>(max_mismatches + 1),
@@ -587,16 +596,18 @@ std::vector<GaplessExtension> GaplessExtender::extend(cluster_type& cluster, con
                         next.old_score = next.internal_score;
                     }
                     set_score(next, this->aligner);
+                    num_extensions += next.state.size();
                     extensions.push(std::move(next));
-                    found_extension = true;
                     return true;
                 });
-                if (!found_extension) {
+                // We could not extend all threads in 'curr' to the right. The unextended ones
+                // may have different left extensions, so we must consider 'curr' right-maximal.
+                if (num_extensions < curr.state.size()) {
                     curr.right_maximal = true;
                     curr.old_score = curr.internal_score;
-                } else {
-                    continue;
+                    extensions.push(std::move(curr));
                 }
+                continue;
             }
 
             // Case 2: Extend to the left.
@@ -657,8 +668,8 @@ std::vector<GaplessExtension> GaplessExtender::extend(cluster_type& cluster, con
         }
     }
 
-    // If we have a good enough full-length alignment, return the best two sufficiently
-    // distinct full-length alingments.
+    // If we have a good enough full-length alignment, return the best sufficiently
+    // distinct full-length alignments.
     if (best_alignment < result.size() && result[best_alignment].internal_score <= max_mismatches) {
         handle_full_length(*cache, result, overlap_threshold);
         find_mismatches(sequence, *cache, result);
@@ -689,6 +700,12 @@ std::vector<GaplessExtension> GaplessExtender::extend(cluster_type& cluster, con
 
 //------------------------------------------------------------------------------
 
+bool GaplessExtender::full_length_extensions(const std::vector<GaplessExtension>& result, size_t max_mismatches) {
+    return (result.size() > 0 && result.front().full() && result.front().mismatches() <= max_mismatches);
+}
+
+//------------------------------------------------------------------------------
+
 struct state_hash {
     size_t operator()(const gbwt::BidirectionalState& state) const {
         size_t result = wang_hash_64(state.forward.node);
@@ -889,4 +906,20 @@ void GaplessExtender::transform_alignment(Alignment& aln, const std::vector<std:
 
 //------------------------------------------------------------------------------
 
+void GaplessExtender::init_mask() {
+    this->mask = std::vector<char>(256, 'X');
+    this->mask[static_cast<size_t>('A')] = 'A';
+    this->mask[static_cast<size_t>('C')] = 'C';
+    this->mask[static_cast<size_t>('G')] = 'G';
+    this->mask[static_cast<size_t>('T')] = 'T';
+}
+
+void GaplessExtender::mask_sequence(std::string& sequence) const {
+    for (char& c : sequence) {
+        c = this->mask[static_cast<size_t>(c)];
+    }
+}
+
+//------------------------------------------------------------------------------
+
 } // namespace vg
