@@ -52,28 +52,79 @@ inline gbwt::vector_type path_to_gbwt(const Path& path) {
     return result;
 }
 
+/// Extract a path as a GBWT path.
+gbwt::vector_type extract_as_gbwt_path(const PathHandleGraph& graph, const std::string& path_name);
+
 // Find all predecessor nodes of the path, ignoring self-loops.
-inline gbwt::vector_type path_predecessors(const HandleGraph& graph, const Path& path) {
-    gbwt::vector_type result;
-    if (path.mapping_size() == 0) {
-        return result;
-    }
+gbwt::vector_type path_predecessors(const PathHandleGraph& graph, const std::string& path_name);
 
-    vg::id_t first_node = path.mapping(0).position().node_id();
-    bool is_reverse = path.mapping(0).position().is_reverse();
-    
-#ifdef debug
-    cerr << "Look for predecessors of node " << first_node << " " << is_reverse << " which is first in alt path" << endl;
-#endif
+//------------------------------------------------------------------------------
 
-    graph.follow_edges(graph.get_handle(first_node), !is_reverse, [&] (handle_t next) {
-            if (graph.get_id(next) != first_node) {
-                result.push_back(gbwt::Node::encode(graph.get_id(next), graph.get_is_reverse(next)));
-            }
-        });
+// GBWT construction helpers.
 
-    return result;
-}
+/// Determine the node width in bits for the GBWT nodes based on the given graph.
+gbwt::size_type gbwt_node_width(const HandleGraph& graph);
+
+/// Finish GBWT construction and optionally print the metadata.
+void finish_gbwt_constuction(gbwt::GBWTBuilder& builder,
+    const std::vector<std::string>& sample_names,
+    const std::vector<std::string>& contig_names,
+    size_t haplotype_count, bool print_metadata,
+    const std::string& header = "GBWT");
+
+//------------------------------------------------------------------------------
+
+/// Load a compressed GBWT from the file.
+void load_gbwt(const std::string& filename, gbwt::GBWT& index, bool show_progress = false);
+
+/// Load a dynamic GBWT from the file.
+void load_gbwt(const std::string& filename, gbwt::DynamicGBWT& index, bool show_progress = false);
+
+/**
+ * Helper class that stores either a GBWT or a DynamicGBWT and loads them from a file
+ * or converts between them when necessary.
+ */
+struct GBWTHandler {
+    enum index_type { index_none, index_compressed, index_dynamic };
+
+    /// Compressed GBWT.
+    gbwt::GBWT compressed;
+
+    /// Dynamic GBWT.
+    gbwt::DynamicGBWT dynamic;
+
+    /// Which index is in use.
+    index_type in_use = index_none;
+
+    /// The in-memory indexes are backed by this file.
+    std::string filename;
+
+    /// Print progress information to stderr when loading/converting indexes.
+    bool show_progress = false;
+
+    /// Switch to a compressed GBWT, converting it from the dynamic GBWT or reading it
+    /// from a file if necessary.
+    void use_compressed();
+
+    /// Switch to a dynamic GBWT, converting it from the compressed GBWT or reading it
+    /// from a file if necessary.
+    void use_dynamic();
+
+    /// Start using this compressed GBWT. Clears the index used as the argument.
+    void use(gbwt::GBWT& new_index);
+
+    /// Start using this dynamic GBWT. Clears the index used as the argument.
+    void use(gbwt::DynamicGBWT& new_index);
+
+    /// The GBWT is no longer backed by a file.
+    void unbacked();
+
+    /// Serialize the in-memory index to this file and start using it as the backing file.
+    void serialize(const std::string& new_filename);
+
+    /// Clear the in-memory index.
+    void clear();
+};
 
 //------------------------------------------------------------------------------
 
