@@ -4,21 +4,21 @@
 /**
  * \file gfa.hpp
  *
- * Defines GFA I/O algorithms for VG graphs.
+ * Defines GFA I/O algorithms for PathHandleGraphs graphs.
  *
  * Includes an algorithm for converting from GFA, including non-perfect-match
  * edge overlaps and edges that specify containment of one node in another, to
  * a blunt-ended VG.
  */
 
-#include "vg.hpp"
+#include "handle.hpp"
 
 namespace vg {
 
 using namespace std;
 
 /// Export the given VG graph to the given GFA file.
-void graph_to_gfa(const VG* graph, ostream& out);
+void graph_to_gfa(const unique_ptr<PathHandleGraph>& graph, ostream& out);
 
 
 }
