@@ -2,6 +2,7 @@
 #define VG_MIN_DISTANCE_HPP_INCLUDED
 
 #include <unordered_set>
+#include <jansson.h>
 
 #include "snarls.hpp"
 #include "hash_map.hpp"
@@ -59,6 +60,12 @@ class MinimumDistanceIndex {
     int64_t max_distance(pos_t pos1, pos_t pos2) const;
 
 
+    ///Get the start node (id and orientation pointing  into the snarl) of the
+    //snarl that this point into and a bool is_trivial_snarl
+    //Returns <0, false, false> if this doesn't point into a snarl
+    tuple<id_t, bool, bool> into_which_snarl(id_t node_id, bool reverse) const;
+
+
     //Given an alignment to a graph and a range, find the set of nodes in the
     //graph for which the minimum distance from the position to any position
     //in the node is within the given distance range
@@ -98,8 +105,11 @@ class MinimumDistanceIndex {
         return static_cast<int64_t>(std::min(static_cast<uint64_t>(x), static_cast<uint64_t>(y)));
     }
 
+    ///Write snarls out to stout
+    void write_snarls_to_json();
+
     ///print the distance index for debugging
-    void print_self();
+    void print_self() const;
     // Prints the number of nodes in each snarl netgraph and number of snarls in each chain
     void print_snarl_stats();
 
@@ -167,7 +177,10 @@ class MinimumDistanceIndex {
             void insert_distance(size_t start, size_t end, int64_t dist);
 
 
-            void print_self();
+            bool is_trivial_snarl() const;
+
+            void print_self() const;
+            json_t* snarl_to_json();
 
         protected:
  
@@ -265,7 +278,8 @@ class MinimumDistanceIndex {
                 return prefix_sum[prefix_sum.size() - 1] - 1;
             }
 
-            void print_self();
+            void print_self() const;
+            json_t* chain_to_json();
 
         protected:
 
