@@ -11,6 +11,7 @@
 #include <stdio.h>
 #include <vector>
 #include <unordered_map>
+#include <algorithm>
 
 #include "structures/rank_pairing_heap.hpp"
 
@@ -62,21 +63,28 @@ namespace vg {
         /// Produces a graph with reachability edges. Assumes that the cluster
         /// is sorted by primarily length and secondarily lexicographically by
         /// read interval.
-        MultipathAlignmentGraph(const HandleGraph& graph, const MultipathMapper::memcluster_t& hits,
+        /// If a hit fails to be walked ouut in the graph, it is removed from hits.
+        MultipathAlignmentGraph(const HandleGraph& graph, MultipathMapper::memcluster_t& hits,
                                 const function<pair<id_t, bool>(id_t)>& project,
                                 const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans,
-                                size_t max_branch_trim_length = 0, gcsa::GCSA* gcsa = nullptr);
+                                vector<size_t>& path_node_provenance,
+                                size_t max_branch_trim_length = 0, gcsa::GCSA* gcsa = nullptr,
+                                const MultipathMapper::match_fanouts_t* fanout_breaks = nullptr);
                                 
         /// Same as the previous constructor, but construct injection_trans implicitly and temporarily.
-        MultipathAlignmentGraph(const HandleGraph& graph, const MultipathMapper::memcluster_t& hits,
+        MultipathAlignmentGraph(const HandleGraph& graph, MultipathMapper::memcluster_t& hits,
                                 const unordered_map<id_t, pair<id_t, bool>>& projection_trans,
-                                size_t max_branch_trim_length = 0, gcsa::GCSA* gcsa = nullptr);
+                                vector<size_t>& path_node_provenance,
+                                size_t max_branch_trim_length = 0, gcsa::GCSA* gcsa = nullptr,
+                                const MultipathMapper::match_fanouts_t* fanout_breaks = nullptr);
         
         /// Same as the previous constructor, but construct injection_trans implicitly and temporarily
         /// using a lambda for a projector
-        MultipathAlignmentGraph(const HandleGraph& graph, const MultipathMapper::memcluster_t& hits,
+        MultipathAlignmentGraph(const HandleGraph& graph, MultipathMapper::memcluster_t& hits,
                                 const function<pair<id_t, bool>(id_t)>& project,
-                                size_t max_branch_trim_length = 0, gcsa::GCSA* gcsa = nullptr);
+                                vector<size_t>& path_node_provenance,
+                                size_t max_branch_trim_length = 0, gcsa::GCSA* gcsa = nullptr,
+                                const MultipathMapper::match_fanouts_t* fanout_breaks = nullptr);
         
         /// Construct a graph of the reachability between aligned chunks in a linearized
         /// path graph. Produces a graph with reachability edges.
@@ -96,18 +104,22 @@ namespace vg {
                                 const Alignment& alignment, const function<pair<id_t, bool>(id_t)>& project, bool realign_Ns = true,
                                 bool preserve_tail_anchors = false);
         
-        /// Make a multipath alignment graph using the path of a single-path alignment
-        MultipathAlignmentGraph(const HandleGraph& graph, const Alignment& alignment, SnarlManager& snarl_manager, size_t max_snarl_cut_size,
+        /// Make a multipath alignment graph using the path of a single-path alignment. Only
+        /// one of snarl_manager and dist_index need be supplied.
+        MultipathAlignmentGraph(const HandleGraph& graph, const Alignment& alignment, SnarlManager* snarl_manager,
+                                MinimumDistanceIndex* dist_index, size_t max_snarl_cut_size,
                                 const function<pair<id_t, bool>(id_t)>& project,
                                 const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans);
         
         /// Same as the previous constructor, but construct injection_trans implicitly and temporarily
-        MultipathAlignmentGraph(const HandleGraph& graph, const Alignment& alignment, SnarlManager& snarl_manager, size_t max_snarl_cut_size,
+        MultipathAlignmentGraph(const HandleGraph& graph, const Alignment& alignment, SnarlManager* snarl_manager,
+                                MinimumDistanceIndex* dist_index, size_t max_snarl_cut_size,
                                 const unordered_map<id_t, pair<id_t, bool>>& projection_trans);
         
         /// Same as the previous constructor, but construct injection_trans implicitly and temporarily
         /// using a function instead of a map
-        MultipathAlignmentGraph(const HandleGraph& graph, const Alignment& alignment, SnarlManager& snarl_manager, size_t max_snarl_cut_size,
+        MultipathAlignmentGraph(const HandleGraph& graph, const Alignment& alignment, SnarlManager* snarl_manager,
+                                MinimumDistanceIndex* dist_index, size_t max_snarl_cut_size,
                                 const function<pair<id_t, bool>(id_t)>& project);
         
         ~MultipathAlignmentGraph();
@@ -122,7 +134,10 @@ namespace vg {
         /// end of the alignment sequence.
         void trim_hanging_indels(const Alignment& alignment, bool trim_Ns = true, bool preserve_tail_anchors = false);
         
-        /// Removes all transitive edges from graph (reduces to minimum equivalent graph).
+        /// Removes all transitive edges from graph (reduces to minimum equivalent graph),
+        /// except for edges between path nodes that abut either on the graph or read. These
+        /// edges often correspond to overlap breaks in low complexity sequence, so retaining
+        /// them improves alignment in low-complexity regions like STR expansions.
         /// Note: reorders internal representation of adjacency lists.
         /// Reachability edges must be in the graph.
         void remove_transitive_edges(const vector<size_t>& topological_order);
@@ -130,7 +145,9 @@ namespace vg {
         /// Removes nodes and edges that are not part of any path that has an estimated score
         /// within some amount of the highest scoring path. Reachability edges must be present.
         void prune_to_high_scoring_paths(const Alignment& alignment, const GSSWAligner* aligner,
-                                         double max_suboptimal_score_ratio, const vector<size_t>& topological_order);
+                                         double max_suboptimal_score_ratio, const vector<size_t>& topological_order,
+                                         function<pair<id_t, bool>(id_t)>& translator,
+                                         vector<size_t>& path_node_provenance);
         
         /// Clear reachability edges, so that add_reachability_edges can be run
         /// (possibly after modifying the graph).
@@ -142,9 +159,10 @@ namespace vg {
         
         /// Cut the interior of snarls out of anchoring paths (and split
         /// alignment nodes accordingly) unless they are longer than the max
-        /// cut size. Reachability edges must be cleared.
-        void resect_snarls_from_paths(SnarlManager* cutting_snarls, const function<pair<id_t, bool>(id_t)>& project,
-                                      int64_t max_snarl_cut_size = 5);
+        /// cut size. Snarls can be stored either in a SnarlManager or a
+        /// MinimumDistanceIndex (only one need be supplied).
+        void resect_snarls_from_paths(SnarlManager* cutting_snarls, MinimumDistanceIndex* dist_index,
+                                      const function<pair<id_t, bool>(id_t)>& project, int64_t max_snarl_cut_size = 5);
         
         /// Do some exploratory alignments of the tails of the graph, outside
         /// the outermost existing anchors, and define new anchoring paths from
@@ -163,7 +181,8 @@ namespace vg {
         /// Add edges between reachable nodes and split nodes at overlaps
         void add_reachability_edges(const HandleGraph& vg,
                                     const function<pair<id_t, bool>(id_t)>& project,
-                                    const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans);
+                                    const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans,
+                                    vector<size_t>* path_node_provenance = nullptr);
                                     
         /// Do intervening and tail alignments between the anchoring paths and
         /// store the result in a multipath_alignment_t. Reachability edges must
@@ -204,7 +223,9 @@ namespace vg {
         vector<vector<id_t>> get_connected_components() const;
         
         /// Does the multipath alignment graph have any nodes?
-        bool empty();
+        bool empty() const;
+        
+        size_t size() const;
         
     private:
         
@@ -236,14 +257,27 @@ namespace vg {
                                      const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans);
         
         /// Walk out MEMs into match nodes and filter out redundant sub-MEMs
-        void create_match_nodes(const HandleGraph& graph, const MultipathMapper::memcluster_t& hits,
+        void create_match_nodes(const HandleGraph& graph, MultipathMapper::memcluster_t& hits,
                                 const function<pair<id_t, bool>(id_t)>& project,
-                                const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans);
+                                const unordered_multimap<id_t, pair<id_t, bool>>& injection_trans,
+                                vector<size_t>& path_node_provenance,
+                                int64_t max_branch_trim_length,
+                                const MultipathMapper::match_fanouts_t* fanout_breaks);
+        
+        /// If path nodes partially overlap, merge the sections that overlap into a single path node
+        void merge_partially_redundant_match_nodes(const unordered_map<int64_t, vector<int64_t>>& node_matches,
+                                                   vector<size_t>& path_node_provenance);
+        
+        void jitter_homopolymer_ends(const HandleGraph& graph,
+                                     vector<size_t>& path_node_provenance,
+                                     const MultipathMapper::memcluster_t& hits,
+                                     int64_t max_branch_trim_length);
         
         /// Identifies runs of exact matches that are sub-maximal because they hit the order of the GCSA
         /// index and merges them into a single node, assumes that match nodes are sorted by length and
         /// then lexicographically by read interval, does not update edges
-        void collapse_order_length_runs(const HandleGraph& graph, gcsa::GCSA* gcsa);
+        void collapse_order_length_runs(const HandleGraph& graph, gcsa::GCSA* gcsa,
+                                        vector<size_t>& path_node_provenance);
         
         /// Reorders adjacency list representation of edges so that they follow the indicated
         /// ordering of their target nodes
@@ -253,6 +287,10 @@ namespace vg {
         /// the square root of the tail length)
         int64_t pessimistic_tail_gap(int64_t tail_length, double multiplier);
         
+        /// Returns true if we're pointing into a snarl that we want to cut out of paths
+        bool into_cutting_snarl(id_t node_id, bool is_rev,
+                                SnarlManager* snarl_manager, MinimumDistanceIndex* dist_index);
+        
         /// Generate alignments of the tails of the query sequence, beyond the
         /// sources and sinks. The Alignment passed *must* be the one that owns
         /// the sequence we are working on. Returns a map from tail
