@@ -2,6 +2,7 @@
 #define VG_SIMULATOR_HPP_INCLUDED
 
 #include <iostream>
+#include <iomanip>
 #include <map>
 #include <unordered_map>
 #include <vector>
@@ -15,7 +16,7 @@
 #include "position.hpp"
 #include "statistics.hpp"
 #include "lru_cache.h"
-#include "json2pb.h"
+#include "vg/io/json2pb.h"
 #include "algorithms/subgraph.hpp"
 #include "algorithms/alignment_path_offsets.hpp"
 #include "algorithms/next_pos_chars.hpp"
@@ -54,11 +55,17 @@ public:
     vector<string> source_paths;
     vg::discrete_distribution<> path_sampler; // draw an index in source_paths
     size_t total_seq_length = 0;
+    
+    /// Make a Sampler to sample from the given graph.
+    /// If sampling from particular paths, source_paths should contain their
+    /// names, and source_path_ploidies should either be empty or contain a
+    /// ploidy value for each source path.
     inline Sampler(PathPositionHandleGraph* x,
             int seed = 0,
             bool forward_only = false,
             bool allow_Ns = false,
             const vector<string>& source_paths = {},
+            const vector<double>& source_path_ploidies = {},
             const vector<pair<string, double>>& transcript_expressions = {},
             const vector<tuple<string, string, size_t>>& haplotype_transcripts = {})
         : xgidx(x),
@@ -76,15 +83,17 @@ public:
             seed = time(NULL);
         }
         rng.seed(seed);
-        set_source_paths(source_paths, transcript_expressions, haplotype_transcripts);
+        set_source_paths(source_paths, source_path_ploidies, transcript_expressions, haplotype_transcripts);
     }
 
-    /// Make a path sampling distribution based on relative lengths or on transcript expressions
-    /// (at most one should be non-empty)
-    /// If providing a transcript expression profile, can optionally provide a non-empty vector
-    /// of haplotype transcripts to translate between the embedded path names and the transcript
-    /// names in the expression profile
+    /// Make a path sampling distribution based on relative lengths (weighted
+    /// by ploidy) or on transcript expressions. (At most one of source_paths and
+    /// expressions should be non-empty.) If providing a transcript expression
+    /// profile, can optionally provide a non-empty vector of haplotype
+    /// transcripts to translate between the embedded path names and the
+    /// transcript names in the expression profile.
     void set_source_paths(const vector<string>& source_paths,
+                          const vector<double>& source_path_ploidies,
                           const vector<pair<string, double>>& transcript_expressions,
                           const vector<tuple<string, string, size_t>>& haplotype_transcripts);
 
@@ -158,6 +167,7 @@ public:
                  const string& ngs_paired_fastq_file = "",
                  bool interleaved_fastq = false,
                  const vector<string>& source_paths = {},
+                 const vector<double>& source_path_ploidies = {},
                  const vector<pair<string, double>>& transcript_expressions = {},
                  const vector<tuple<string, string, size_t>>& haplotype_transcripts = {},
                  double substition_polymorphism_rate = 0.001,
@@ -176,6 +186,9 @@ public:
     /// Sample a pair of reads an alignments
     pair<Alignment, Alignment> sample_read_pair();
     
+    /// Open up a stream to output read positions to
+    void connect_to_position_file(const string& filename);
+    
 private:
     template<class From, class To>
     class MarkovDistribution {
@@ -199,7 +212,7 @@ private:
         unordered_map<From, vector<size_t>> cond_distrs;
         
     };
-    
+        
     NGSSimulator(void) = delete;
     
     /// DNA alphabet
@@ -228,23 +241,27 @@ private:
     /// the iteration and update of curr_pos) in path node. Otherwise, in whole
     /// graph mode, they are ignored and curr_pos is used to traverse the graph
     /// directly.
-    void sample_read_internal(Alignment& aln, size_t& offset, bool& is_reverse, pos_t& curr_pos,
+    void sample_read_internal(Alignment& aln, int64_t& offset, bool& is_reverse, pos_t& curr_pos,
                               const string& source_path);
     
     /// Return the index of a path if using source_paths or else numeric_limits<size_t>::max()
     size_t sample_path();
     
+    /// Ouput a sampled position to the path position file
+    void register_sampled_position(const Alignment& aln, const string& path_name,
+                                   size_t offset, bool is_reverse);
+    
     /// Sample an appropriate starting position according to the mode. Updates the arguments.
     /// Providing a negative number for fragment length indicates no fragment length restrictions.
     void sample_start_pos(const size_t& source_path_idx, const int64_t& fragment_length,
-                          size_t& offset, bool& is_reverse, pos_t& pos);
+                          int64_t& offset, bool& is_reverse, pos_t& pos);
     
     /// Get a random position in the graph
     pos_t sample_start_graph_pos();
     /// Get a random position along the source path. Enforce fragment length restrictions if argument
     /// is positive.
-    tuple<size_t, bool, pos_t> sample_start_path_pos(const size_t& source_path_idx,
-                                                     const int64_t& fragment_length);
+    tuple<int64_t, bool, pos_t> sample_start_path_pos(const size_t& source_path_idx,
+                                                      const int64_t& fragment_length);
     
     /// Get an unclashing read name
     string get_read_name();
@@ -252,20 +269,20 @@ private:
     /// Move forward one position in either the source path or the graph,
     /// depending on mode. Update the arguments. Return true if we can't because
     /// we hit a tip or false otherwise
-    bool advance(size_t& offset, bool& is_reverse, pos_t& pos, char& graph_char, const string& source_path);
+    bool advance(int64_t& offset, bool& is_reverse, pos_t& pos, char& graph_char, const string& source_path);
     /// Move forward a certain distance in either the source path or the graph,
     /// depending on mode. Update the arguments. Return true if we can't because
     /// we hit a tip or false otherwise
-    bool advance_by_distance(size_t& offset, bool& is_reverse, pos_t& pos, size_t distance,
+    bool advance_by_distance(int64_t& offset, bool& is_reverse, pos_t& pos, int64_t distance,
                              const string& source_path);
     
     /// Move forward one position in the source path, return true if we can't
     /// because we hit a tip or false otherwise
-    bool advance_on_path(size_t& offset, bool& is_reverse, pos_t& pos, char& graph_char,
+    bool advance_on_path(int64_t& offset, bool& is_reverse, pos_t& pos, char& graph_char,
                          const string& source_path);
     /// Move forward a certain distance in the source path, return true if we
     /// can't because we hit a tip or false otherwise
-    bool advance_on_path_by_distance(size_t& offset, bool& is_reverse, pos_t& pos, size_t distance,
+    bool advance_on_path_by_distance(int64_t& offset, bool& is_reverse, pos_t& pos, int64_t distance,
         const string& source_path);
     
     /// Move forward one position in the graph along a random path, return true if we can't
@@ -273,13 +290,13 @@ private:
     bool advance_on_graph(pos_t& pos, char& graph_char);
     /// Move forward a certain distance in the graph along a random path, return true if we
     /// can't because we hit a tip or false otherwise
-    bool advance_on_graph_by_distance(pos_t& pos, size_t distance);
+    bool advance_on_graph_by_distance(pos_t& pos, int64_t distance);
     
     /// Mask out bases with 'N's if the mask is true
     void apply_N_mask(string& sequence, const vector<bool>& n_mask);
     
     /// Returns the position a given distance from the end of the path, walking backwards
-    pos_t walk_backwards(const Path& path, size_t distance);
+    pos_t walk_backwards(const Path& path, int64_t distance);
     /// Add a deletion to the alignment
     void apply_deletion(Alignment& aln, const pos_t& pos);
     /// Add an insertion to the alignment
@@ -287,7 +304,7 @@ private:
     /// Add a match/mismatch to the alignment
     void apply_aligned_base(Alignment& aln, const pos_t& pos, char graph_char, char read_char);
     
-    /// Memo for Phred -> probability conversion
+    /// Memo for pre-multiplied Phred -> probability conversion
     vector<double> phred_prob;
     
     /// A Markov distribution for each read position indicating quality and whether the base is an 'N'
@@ -322,6 +339,8 @@ private:
     
     /// Restrict reads to just these paths (path-only mode) if nonempty.
     vector<string> source_paths;
+    
+    ofstream position_file;
 };
     
 /**
