@@ -15,7 +15,6 @@
 
 #include "../vg.hpp"
 #include "../variant_adder.hpp"
-#include "../algorithms/copy_graph.hpp"
 
 #include <vg/io/vpkg.hpp>
 
@@ -178,7 +177,7 @@ int main_add(int argc, char** argv) {
         if (vg_graph == nullptr) {
             // Copy instead.
             vg_graph = new vg::VG();
-            algorithms::copy_path_handle_graph(graph.get(), vg_graph);
+            handlealgs::copy_path_handle_graph(graph.get(), vg_graph);
             // Give the unique_ptr ownership and delete the graph we loaded.
             graph.reset(vg_graph);
             // Make sure the paths are all synced up
