@@ -21,10 +21,8 @@
 #include "../region.hpp"
 #include "../haplotype_extracter.hpp"
 #include "../algorithms/sorted_id_ranges.hpp"
-#include "../algorithms/weakly_connected_components.hpp"
 #include <bdsg/overlays/overlay_helper.hpp>
 #include "../io/save_handle_graph.hpp"
-#include "../algorithms/copy_graph.hpp"
 
 using namespace std;
 using namespace vg;
@@ -45,7 +43,7 @@ void help_chunk(char** argv) {
          << endl
          << "options:" << endl
          << "    -x, --xg-name FILE       use this graph or xg index to chunk subgraphs" << endl
-         << "    -G, --gbwt-name FILE     use this GBWT haplotype index for haplotype extraction" << endl
+         << "    -G, --gbwt-name FILE     use this GBWT haplotype index for haplotype extraction (for -T)" << endl
          << "    -a, --gam-name FILE      chunk this gam file instead of the graph (multiple allowed)" << endl
          << "    -g, --gam-and-graph      when used in combination with -a, both gam and graph will be chunked" << endl 
          << "path chunking:" << endl
@@ -74,7 +72,7 @@ void help_chunk(char** argv) {
          << "    -T, --trace              trace haplotype threads in chunks (and only expand forward from input coordinates)." << endl
          << "                             Produces a .annotate.txt file with haplotype frequencies for each chunk." << endl 
          << "    -f, --fully-contained    only return GAM alignments that are fully contained within chunk" << endl
-         << "    -O, --output-fmt         Specifiy output format (vg, pg, hg).  [VG]" << endl
+         << "    -O, --output-fmt         Specify output format (vg, pg, hg).  [VG]" << endl
          << "    -t, --threads N          for tasks that can be done in parallel, use this many threads [1]" << endl
          << "    -h, --help" << endl;
 }
@@ -301,7 +299,7 @@ int main_chunk(int argc, char** argv) {
     // check the output format
     std::transform(output_format.begin(), output_format.end(), output_format.begin(), ::tolower);
     if (!vg::io::valid_output_format(output_format)) {
-        cerr << "error[vg chunk]: invalid ouput format" << endl;
+        cerr << "error[vg chunk]: invalid output format" << endl;
         return 1;
     }
 
@@ -524,7 +522,7 @@ int main_chunk(int argc, char** argv) {
     vector<unordered_set<nid_t>> component_ids; 
     if (components == true && regions.empty()) {
         // no regions given, we find our components from scratch and make some dummy regions
-        component_ids = algorithms::weakly_connected_components(graph);
+        component_ids = handlealgs::weakly_connected_components(graph);
         for (int i = 0; i < component_ids.size(); ++i) {
             Region region;
             region.seq = "";
@@ -655,11 +653,11 @@ int main_chunk(int argc, char** argv) {
                 // our graph is not in vg format.  covert it, extend it, convert it back
                 // this can eventually be avoided by handlifying the haplotype tracer
                 VG vg;
-                algorithms::copy_path_handle_graph(subgraph.get(), &vg);
+                handlealgs::copy_path_handle_graph(subgraph.get(), &vg);
                 subgraph.reset();
                 vg.extend(g);
                 subgraph = vg::io::new_output_graph<MutablePathMutableHandleGraph>(output_format);
-                algorithms::copy_path_handle_graph(&vg, subgraph.get());
+                handlealgs::copy_path_handle_graph(&vg, subgraph.get());
             }
         }
 
