@@ -17,7 +17,6 @@
 #include "subcommand.hpp"
 
 #include "../handle.hpp"
-#include "../algorithms/copy_graph.hpp"
 #include "../vg.hpp"
 
 using namespace std;
@@ -132,7 +131,7 @@ int main_combine(int argc, char** argv) {
             VG* vg_graph = dynamic_cast<vg::VG*>(graph.get());
             if (vg_graph == nullptr) {
                 vg_graph = new vg::VG();
-                algorithms::copy_path_handle_graph(graph.get(), vg_graph);
+                handlealgs::copy_path_handle_graph(graph.get(), vg_graph);
                 // Give the unique_ptr ownership and delete the graph we loaded.
                 graph.reset(vg_graph);
                 // Make sure the paths are all synced up
@@ -158,5 +157,5 @@ int main_combine(int argc, char** argv) {
 }
 
 // Register subcommand
-static Subcommand vg_combine("combine", "merge multiple graph files together", main_combine);
+static Subcommand vg_combine("combine", "merge multiple graph files together", DEPRECATED, main_combine);
 
