@@ -2,12 +2,11 @@
 #include "../vg.hpp"
 #include "../utility.hpp"
 #include "xg.hpp"
-#include "../algorithms/copy_graph.hpp"
 #include "../algorithms/gfa_to_handle.hpp"
 #include "../io/save_handle_graph.hpp"
 #include <vg/io/stream.hpp>
 #include <vg/io/vpkg.hpp>
-#include "../alignment_emitter.hpp"
+#include <vg/io/alignment_emitter.hpp>
 
 #include "bdsg/packed_graph.hpp"
 #include "bdsg/hash_graph.hpp"
@@ -18,6 +17,7 @@
 
 using namespace vg;
 using namespace vg::subcommand;
+using namespace vg::io;
 
 void help_convert(char** argv) {
     cerr << "usage: " << argv[0] << " convert [options] <input-graph>" << endl
@@ -81,6 +81,7 @@ int main_convert(int argc, char** argv) {
             return 0;
         case 'g':
             input_gfa = true;
+            break;
         case 'v':
             output_format = "vg";
             break;
@@ -136,8 +137,8 @@ int main_convert(int argc, char** argv) {
             input_graph = vg::io::VPKG::load_one<HandleGraph>(in);
         });
 
-        unique_ptr<AlignmentEmitter> emitter = get_alignment_emitter("-", gam_to_gaf ? "GAF" : "GAM", {}, get_thread_count(),
-                                                                     input_graph.get());
+        unique_ptr<AlignmentEmitter> emitter = get_non_hts_alignment_emitter("-", gam_to_gaf ? "GAF" : "GAM", {}, get_thread_count(),
+                                                                             input_graph.get());
         std::function<void(Alignment&)> lambda = [&] (Alignment& aln) {
             emitter->emit_singles({aln});
         };                
@@ -188,6 +189,9 @@ int main_convert(int argc, char** argv) {
             dynamic_cast<xg::XG*>(output_graph.get())->from_gfa(input_stream_name);
         }
         else {
+            if (input_stream_name == "-") {
+                cerr << "warning [vg convert]: Converting a GFA to an indexed sequence graph from piped input can be memory intensive because the entire GFA must be loaded into memory. If memory usage is too high, consider serializing the GFA to disk to allow multiple passes over the file." << endl;
+            }
             try {
                 if (output_path_graph != nullptr) {
                     MutablePathMutableHandleGraph* mutable_output_graph = dynamic_cast<MutablePathMutableHandleGraph*>(output_path_graph);
@@ -233,7 +237,7 @@ int main_convert(int argc, char** argv) {
             assert(mutable_output_graph != nullptr);
             // ID hint (currently only for odgi)
             mutable_output_graph->set_id_increment(input_graph->min_node_id());
-            algorithms::copy_path_handle_graph(input_path_graph, mutable_output_graph);
+            handlealgs::copy_path_handle_graph(input_path_graph, mutable_output_graph);
         }
         else {
             if (input_path_graph != nullptr) {
@@ -243,7 +247,7 @@ int main_convert(int argc, char** argv) {
             assert(mutable_output_graph != nullptr);
             // ID hint (currently only for odgi)
             mutable_output_graph->set_id_increment(input_graph->min_node_id());
-            algorithms::copy_handle_graph(input_graph.get(), mutable_output_graph);
+            handlealgs::copy_handle_graph(input_graph.get(), mutable_output_graph);
         }
     }
     
