@@ -21,7 +21,7 @@
 #include "../annotation.hpp"
 #include <vg/io/vpkg.hpp>
 #include <vg/io/stream.hpp>
-#include "../alignment_emitter.hpp"
+#include "../hts_alignment_emitter.hpp"
 #include "../gapless_extender.hpp"
 #include "../minimizer_mapper.hpp"
 #include "../index_manager.hpp"
@@ -223,7 +223,7 @@ struct Range {
 // See https://stackoverflow.com/a/25803794
 
 // In general, things aren't instantiations of things
-template <typename Subject, template<typename...> typename Predicate>
+template <typename Subject, template<typename...> class Predicate>
 struct is_instantiation_of : std::false_type {
 };
 
@@ -293,7 +293,7 @@ void help_giraffe(char** argv) {
     << "  -x, --xg-name FILE            use this xg index or graph" << endl
     << "  -g, --graph-name FILE         use this GBWTGraph" << endl
     << "  -H, --gbwt-name FILE          use this GBWT index" << endl
-    << "  -m, --minimizer-name FILE     use this minimizer index (may repeat)" << endl
+    << "  -m, --minimizer-name FILE     use this minimizer index" << endl
     << "  -d, --dist-name FILE          cluster using this distance index" << endl
     << "  -p, --progress                show progress" << endl
     << "input options:" << endl
@@ -304,16 +304,18 @@ void help_giraffe(char** argv) {
     << "  -M, --max-multimaps INT       produce up to INT alignments for each read [1]" << endl
     << "  -N, --sample NAME             add this sample name" << endl
     << "  -R, --read-group NAME         add this read group" << endl
-    << "  -o, --output-format NAME      output the alignments in NAME format (gam / gaf / json / tsv) [gam]" << endl
+    << "  -o, --output-format NAME      output the alignments in NAME format (gam / gaf / json / tsv / SAM / BAM / CRAM) [gam]" << endl
+    << "  --ref-paths FILE              ordered list of paths in the graph, one per line or HTSlib .dict, for HTSLib @SQ headers" << endl
     << "  -n, --discard                 discard all output alignments (for profiling)" << endl
     << "  --output-basename NAME        write output to a GAM file beginning with the given prefix for each setting combination" << endl
     << "  --report-name NAME            write a TSV of output file and mapping speed to the given file" << endl
+    << "  --show-work                   log how the mapper comes to its conclusions about mapping locations" << endl
     << "algorithm presets:" << endl
     << "  -b, --parameter-preset NAME   set computational parameters (fast / default) [default]" << endl
     << "computational parameters:" << endl
     << "  -c, --hit-cap INT             use all minimizers with at most INT hits [10]" << endl
-    << "  -C, --hard-hit-cap INT        ignore all minimizers with more than INT hits [1500]" << endl
-    << "  -F, --score-fraction FLOAT    select minimizers between hit caps until score is FLOAT of total [0.7]" << endl
+    << "  -C, --hard-hit-cap INT        ignore all minimizers with more than INT hits [500]" << endl
+    << "  -F, --score-fraction FLOAT    select minimizers between hit caps until score is FLOAT of total [0.9]" << endl
     << "  -D, --distance-limit INT      cluster using this distance limit [200]" << endl
     << "  -e, --max-extensions INT      extend up to INT clusters [800]" << endl
     << "  -a, --max-alignments INT      align up to INT extensions [8]" << endl
@@ -325,6 +327,11 @@ void help_giraffe(char** argv) {
     << "  -O, --no-dp                   disable all gapped alignment" << endl
     << "  -r, --rescue-attempts         attempt up to INT rescues per read in a pair [15]" << endl
     << "  -A, --rescue-algorithm NAME   use algorithm NAME for rescue (none / dozeu / gssw / haplotypes) [dozeu]" << endl
+    << "  -L, --max-fragment-length INT assume that fragment lengths should be smaller than INT when estimating the fragment length distribution" << endl
+    << "  --fragment-mean FLOAT         force the fragment length distribution to have this mean (requires --fragment-stdev)" << endl
+    << "  --fragment-stdev FLOAT        force the fragment length distribution to have this standard deviation (requires --fragment-mean)" << endl
+    << "  --paired-distance-limit FLOAT cluster pairs of read using a distance limit FLOAT standard deviations greater than the mean [2.0]" << endl
+    << "  --rescue-subgraph-size FLOAT  search for rescued alignments FLOAT standard deviations greater than the mean [4.0]" << endl
     << "  --track-provenance            track how internal intermediate alignment candidates were arrived at" << endl
     << "  --track-correctness           track if internal intermediate alignment candidates are correct (implies --track-provenance)" << endl
     << "  -t, --threads INT             number of compute threads to use" << endl;
@@ -343,19 +350,24 @@ int main_giraffe(int argc, char** argv) {
     #define OPT_REPORT_NAME 1002
     #define OPT_TRACK_PROVENANCE 1003
     #define OPT_TRACK_CORRECTNESS 1004
+    #define OPT_FRAGMENT_MEAN 1005
+    #define OPT_FRAGMENT_STDEV 1006
+    #define OPT_CLUSTER_STDEV 1007
+    #define OPT_RESCUE_STDEV 1008
+    #define OPT_REF_PATHS 1009
+    #define OPT_SHOW_WORK 1010
     
 
     // initialize parameters with their default options
     
     // This holds and manages finding our indexes.
     IndexManager indexes;
-    vector<string> minimizer_names;
     string output_basename;
     string report_name;
     // How close should two hits be to be in the same cluster?
     Range<size_t> distance_limit = 200;
-    Range<size_t> hit_cap = 10, hard_hit_cap = 1500;
-    Range<double> minimizer_score_fraction = 0.7;
+    Range<size_t> hit_cap = 10, hard_hit_cap = 500;
+    Range<double> minimizer_score_fraction = 0.9;
     bool show_progress = false;
     // Should we try chaining or just give up if we can't find a full length gapless alignment?
     bool do_dp = true;
@@ -387,9 +399,24 @@ int main_giraffe(int argc, char** argv) {
     //Throw away extensions with scores that are this amount below the best
     Range<int> extension_score = 1;
     //Attempt up to this many rescues of reads with no pairs
+    bool forced_rescue_attempts = false;
     Range<int> rescue_attempts = 15;
+    //Don't let distances larger than this contribute to the fragment length distribution
+    size_t fragment_length = 2000;
     // Which rescue algorithm do we use?
     MinimizerMapper::RescueAlgorithm rescue_algorithm = MinimizerMapper::rescue_dozeu;
+    //Did we force the fragment length distribution?
+    bool forced_mean = false;
+    //And if so what is it?
+    double fragment_mean = 0.0;
+    bool forced_stdev = false;
+    double fragment_stdev = 0.0;
+    //How many sdevs to we look out when clustering pairs?
+    double cluster_stdev = 2.0;
+    //How many stdevs do we look out when rescuing? 
+    double rescue_stdev = 4.0;
+    // How many pairs should we be willing to buffer before giving up on fragment length estimation?
+    size_t MAX_BUFFERED_PAIRS = 100000;
     // What sample name if any should we apply?
     string sample_name;
     // What read group if any should we apply?
@@ -400,6 +427,8 @@ int main_giraffe(int argc, char** argv) {
     bool track_provenance = false;
     // Should we track candidate correctness?
     bool track_correctness = false;
+    // Should we log our mapping decision making?
+    bool show_work = false;
 
     // Chain all the ranges and get a function that loops over all combinations.
     auto for_each_combo = distance_limit
@@ -420,7 +449,10 @@ int main_giraffe(int argc, char** argv) {
 
     // Formats for alignment output.
     std::string output_format = "GAM";
-    std::set<std::string> output_formats = { "GAM", "GAF", "JSON", "TSV" };
+    std::set<std::string> output_formats = { "GAM", "GAF", "JSON", "TSV", "SAM", "BAM", "CRAM" };
+
+    // For HTSlib formats, where do we get sequence header info?
+    std::string ref_paths_name;
 
     // Map algorithm names to rescue algorithms
     std::map<std::string, MinimizerMapper::RescueAlgorithm> rescue_algorithms = {
@@ -455,6 +487,7 @@ int main_giraffe(int argc, char** argv) {
             {"sample", required_argument, 0, 'N'},
             {"read-group", required_argument, 0, 'R'},
             {"output-format", required_argument, 0, 'o'},
+            {"ref-paths", required_argument, 0, OPT_REF_PATHS},
             {"discard", no_argument, 0, 'n'},
             {"output-basename", required_argument, 0, OPT_OUTPUT_BASENAME},
             {"report-name", required_argument, 0, OPT_REPORT_NAME},
@@ -473,14 +506,20 @@ int main_giraffe(int argc, char** argv) {
             {"no-dp", no_argument, 0, 'O'},
             {"rescue-attempts", required_argument, 0, 'r'},
             {"rescue-algorithm", required_argument, 0, 'A'},
+            {"paired-distance-limit", required_argument, 0, OPT_CLUSTER_STDEV },
+            {"rescue-subgraph-size", required_argument, 0, OPT_RESCUE_STDEV },
+            {"max-fragment-length", required_argument, 0, 'L' },
+            {"fragment-mean", required_argument, 0, OPT_FRAGMENT_MEAN },
+            {"fragment-stdev", required_argument, 0, OPT_FRAGMENT_STDEV },
             {"track-provenance", no_argument, 0, OPT_TRACK_PROVENANCE},
             {"track-correctness", no_argument, 0, OPT_TRACK_CORRECTNESS},
+            {"show-work", no_argument, 0, OPT_SHOW_WORK},
             {"threads", required_argument, 0, 't'},
             {0, 0, 0, 0}
         };
 
         int option_index = 0;
-        c = getopt_long (argc, argv, "hx:g:H:m:s:d:pG:f:iM:N:R:o:nb:c:C:D:F:e:a:S:u:v:w:Ot:r:A:",
+        c = getopt_long (argc, argv, "hx:g:H:m:s:d:pG:f:iM:N:R:o:nb:c:C:D:F:e:a:S:u:v:w:Ot:r:A:L:",
                          long_options, &option_index);
 
 
@@ -520,8 +559,6 @@ int main_giraffe(int argc, char** argv) {
                     exit(1);
                 }
                 indexes.set_minimizer_override(optarg);
-                // In case we have multiple minimizer indexes, save all the names.
-                minimizer_names.emplace_back(optarg);
                 break;
                 
             case 'd':
@@ -594,6 +631,10 @@ int main_giraffe(int argc, char** argv) {
                     }
                 }
                 break;
+                
+            case OPT_REF_PATHS:
+                ref_paths_name = optarg;
+                break;
 
             case 'n':
                 discard_alignments = true;
@@ -612,7 +653,7 @@ int main_giraffe(int argc, char** argv) {
                 if (param_preset == "fast" ) {
 
                     hit_cap = 10;
-                    hard_hit_cap = 1000;
+                    hard_hit_cap = 500;
                     minimizer_score_fraction = 0.5;
                     max_multimaps = 1;
                     max_extensions = 400;
@@ -745,15 +786,12 @@ int main_giraffe(int argc, char** argv) {
                 
             case 'r':
                 {
+                    forced_rescue_attempts = true;
                     rescue_attempts = parse<Range<int>>( optarg);
                     if (rescue_attempts < 0) {
                         cerr << "error: [vg giraffe] Rescue attempts must be positive" << endl;
                         exit(1);
                     }
-                    if (!paired) {
-                        cerr << "error: [vg giraffe] Rescue can only be done on paired-end reads" << endl;
-                        exit(1);
-                    }
                 }
                 break;
 
@@ -772,6 +810,27 @@ int main_giraffe(int argc, char** argv) {
                 }
                 break;
 
+            case OPT_FRAGMENT_MEAN:
+                forced_mean = true;
+                fragment_mean = parse<double>(optarg);
+                break;
+
+            case OPT_FRAGMENT_STDEV:
+                forced_stdev = true;
+                fragment_stdev = parse<double>(optarg);
+                break;
+            case 'L':
+                fragment_length = parse<size_t>(optarg);
+                break;
+
+            case OPT_CLUSTER_STDEV:
+                cluster_stdev = parse<double>(optarg);
+                break;
+
+            case OPT_RESCUE_STDEV:
+                rescue_stdev = parse<double>(optarg);
+                break;
+
             case OPT_TRACK_PROVENANCE:
                 track_provenance = true;
                 break;
@@ -781,6 +840,10 @@ int main_giraffe(int argc, char** argv) {
                 track_correctness = true;
                 break;
                 
+            case OPT_SHOW_WORK:
+                show_work = true;
+                break;
+                
             case 't':
             {
                 int num_threads = parse<int>(optarg);
@@ -847,8 +910,22 @@ int main_giraffe(int argc, char** argv) {
         rescue_attempts = 0;
         rescue_algorithm = MinimizerMapper::rescue_none;
     }
-
+    
     // Now all the arguments are parsed, so see if they make sense
+    
+    // Decide if we are outputting to an htslib format
+    bool hts_output = (output_format == "SAM" || output_format == "BAM" || output_format == "CRAM");
+
+    if (!ref_paths_name.empty() && !hts_output) {
+        cerr << "warning:[vg giraffe] Reference path file (--ref-paths) is only used when output format (-o) is SAM, BAM, or CRAM." << endl;
+        ref_paths_name = "";
+    }
+    
+    if (output_format != "GAM" && !output_basename.empty()) {
+        cerr << "error:[vg giraffe] Using an output basename (--output-basename) only makes sense for GAM format (-o)" << endl;
+        exit(1);
+    }
+    
     if (!indexes.can_get_gbwtgraph() && !indexes.can_get_graph()) {
         cerr << "error:[vg giraffe] Mapping requires a normal graph (-x) or a GBWTGraph (-g)" << endl;
         exit(1);
@@ -859,6 +936,11 @@ int main_giraffe(int argc, char** argv) {
         exit(1);
     }
     
+    if (hts_output && !indexes.can_get_graph()) {
+        cerr << "error:[vg giraffe] Mapping to a linear format requires a normal graph (-x)" << endl;
+        exit(1);
+    }
+    
     if (!indexes.can_get_gbwt()) {
         cerr << "error:[vg giraffe] Mapping requires a GBWT index (-H)" << endl;
         exit(1);
@@ -890,43 +972,35 @@ int main_giraffe(int argc, char** argv) {
         exit(1);
     }
 
+    if ((forced_mean && ! forced_stdev) || (!forced_mean && forced_stdev)) {
+        cerr << "warning:[vg giraffe] Both a mean and standard deviation must be specified for the fragment length distribution" << endl;
+        cerr << "                   Detecting fragment length distribution automatically" << endl;
+        forced_mean = false;
+        forced_stdev = false;
+        fragment_mean = 0.0;
+        fragment_stdev = 0.0;
+    }
+    if ((forced_mean || forced_stdev || forced_rescue_attempts) && (!paired)) {
+        cerr << "warning:[vg giraffe] Attempting to set paired-end parameters but running in single-end mode" << endl;
+    }
+
     // create in-memory objects
     
     // If we are tracking correctness, we will fill this in with a graph for
     // getting offsets along ref paths.
-    PathPositionHandleGraph* positional_graph = nullptr;
+    PathPositionHandleGraph* path_position_graph = nullptr;
     // One of these will actually own it
     bdsg::PathPositionOverlayHelper overlay_helper;
     shared_ptr<PathHandleGraph> graph;
-    if (track_correctness) {
+    if (track_correctness || hts_output) {
         // Load the base graph
         graph = indexes.get_graph();
         // And make sure it has path position support.
         // Overlay is owned by the overlay_helper, if one is needed.
-        positional_graph = overlay_helper.apply(graph.get());
+        path_position_graph = overlay_helper.apply(graph.get());
     }
 
-    vector<unique_ptr<gbwtgraph::DefaultMinimizerIndex>> minimizer_index_owner;
-    shared_ptr<gbwtgraph::DefaultMinimizerIndex> minimizer_index_ref;
-    vector<gbwtgraph::DefaultMinimizerIndex*> minimizer_indexes;
-    if (minimizer_names.size() > 1) {
-        // Working with multiple minimizer indexes
-        for (const string& minimizer_name : minimizer_names) {
-            if (show_progress) {
-                cerr << "Loading minimizer index " << minimizer_name << endl;
-            }
-            // They need to be owned by this vector
-            minimizer_index_owner.emplace_back(vg::io::VPKG::load_one<gbwtgraph::DefaultMinimizerIndex>(minimizer_name));
-            // But this vector we will use to build the index
-            minimizer_indexes.push_back(minimizer_index_owner.back().get());
-        }
-    } else {
-        // Just one index (or we have to make one).
-        // Store a counted reference to it.
-        minimizer_index_ref = indexes.get_minimizer();
-        // And show a pointer to the mapper
-        minimizer_indexes.push_back(minimizer_index_ref.get());
-    }
+    std::shared_ptr<gbwtgraph::DefaultMinimizerIndex> minimizer_index = indexes.get_minimizer();
 
     // Grab the GBWTGraph
     auto gbwt_graph = indexes.get_gbwtgraph();
@@ -938,7 +1012,11 @@ int main_giraffe(int argc, char** argv) {
     if (show_progress) {
         cerr << "Initializing MinimizerMapper" << endl;
     }
-    MinimizerMapper minimizer_mapper(*gbwt_graph, minimizer_indexes, *distance_index, positional_graph);
+    MinimizerMapper minimizer_mapper(*gbwt_graph, *minimizer_index, *distance_index, path_position_graph);
+    if (forced_mean && forced_stdev) {
+        minimizer_mapper.force_fragment_length_distr(fragment_mean, fragment_stdev);
+    }
+
     
     std::chrono::time_point<std::chrono::system_clock> init = std::chrono::system_clock::now();
     std::chrono::duration<double> init_seconds = init - launch;
@@ -1078,11 +1156,25 @@ int main_giraffe(int argc, char** argv) {
             cerr << "--track-correctness " << endl;
         }
         minimizer_mapper.track_correctness = track_correctness;
+        
+        if (show_progress && show_work) {
+            cerr << "--show-work " << endl;
+        }
+        minimizer_mapper.show_work = show_work;
 
         if (show_progress && paired) {
+            if (forced_mean && forced_stdev) {
+                cerr << "--fragment-mean " << fragment_mean << endl; 
+                cerr << "--fragment-stdev " << fragment_stdev << endl;
+            }
+            cerr << "--paired-distance-limit " << cluster_stdev << endl;
+            cerr << "--rescue-subgraph-size " << rescue_stdev << endl;
             cerr << "--rescue-attempts " << rescue_attempts << endl;
             cerr << "--rescue-algorithm " << algorithm_names[rescue_algorithm] << endl;
         }
+        minimizer_mapper.max_fragment_length = fragment_length;
+        minimizer_mapper.paired_distance_stdevs = cluster_stdev;
+        minimizer_mapper.rescue_subgraph_stdevs = rescue_stdev;
         minimizer_mapper.max_rescue_attempts = rescue_attempts;
         minimizer_mapper.rescue_algorithm = rescue_algorithm;
 
@@ -1100,12 +1192,22 @@ int main_giraffe(int argc, char** argv) {
         std::chrono::time_point<std::chrono::system_clock> all_threads_start;
         
         {
-            // Set up output to an emitter that will handle serialization
-            // Discard alignments if asked to. Otherwise spit them out in the selected format.
+        
+            // Look up all the paths we might need to surject to.
+            vector<path_handle_t> paths;
+            if (hts_output) {
+                // For htslib we need a non-empty list of paths.
+                assert(path_position_graph != nullptr);
+                paths = get_sequence_dictionary(ref_paths_name, *path_position_graph);
+            }
+            
+            // Set up output to an emitter that will handle serialization and surjection.
+            // Unless we want to discard all the alignments in which case do that.
+            // We send along the positional graph when we have it, and otherwise we send the GBWTGraph which is sufficient for GAF output.
             unique_ptr<AlignmentEmitter> alignment_emitter = discard_alignments ?
                 make_unique<NullAlignmentEmitter>() :
-                get_alignment_emitter(output_filename, output_format, {}, thread_count, gbwt_graph.get());
-
+                get_alignment_emitter("-", output_format, paths, thread_count, path_position_graph ? (const HandleGraph*)path_position_graph : (const HandleGraph*)gbwt_graph.get());
+            
 #ifdef USE_CALLGRIND
             // We want to profile the alignment, not the loading.
             CALLGRIND_START_INSTRUMENTATION;
@@ -1138,17 +1240,50 @@ int main_giraffe(int argc, char** argv) {
                     return is_ready;
                 };
                 
+                // Define a way to force the distribution ready
+                auto require_distribution_finalized = [&]() {
+                    if (!minimizer_mapper.fragment_distr_is_finalized()){
+                        cerr << "warning[vg::giraffe]: Finalizing fragment length distribution before reaching maximum sample size" << endl;
+                        cerr << "                      mapped " << minimizer_mapper.get_fragment_length_sample_size() 
+                             << " reads single ended with " << ambiguous_pair_buffer.size() << " pairs of reads left unmapped" << endl;
+                        cerr << "                      mean: " << minimizer_mapper.get_fragment_length_mean() << ", stdev: " 
+                             << minimizer_mapper.get_fragment_length_stdev() << endl;
+                        minimizer_mapper.finalize_fragment_length_distr();
+                    }
+                };
+                
                 // Define how to align and output a read pair, in a thread.
                 auto map_read_pair = [&](Alignment& aln1, Alignment& aln2) {
                     
                     pair<vector<Alignment>, vector<Alignment>> mapped_pairs = minimizer_mapper.map_paired(aln1, aln2, ambiguous_pair_buffer);
                     if (!mapped_pairs.first.empty() && !mapped_pairs.second.empty()) {
                         //If we actually tried to map this paired end
-
-                        alignment_emitter->emit_mapped_pair(std::move(mapped_pairs.first), std::move(mapped_pairs.second));
+                        
+                        // Work out whether it could be properly paired or not, if that is relevant.
+                        // If we're here, let the read be properly paired in
+                        // HTSlib terms no matter how far away it is in linear
+                        // space (on the same contig), because it went into
+                        // pair distribution estimation.
+                        // TODO: The semantics are weird here. 0 means
+                        // "properly paired at any distance" and
+                        // numeric_limits<int64_t>::max() doesn't.
+                        int64_t tlen_limit = 0;
+                        if (hts_output && minimizer_mapper.fragment_distr_is_finalized()) {
+                             tlen_limit = minimizer_mapper.get_fragment_length_mean() + 6 * minimizer_mapper.get_fragment_length_stdev();
+                        }
+                        // Emit it
+                        alignment_emitter->emit_mapped_pair(std::move(mapped_pairs.first), std::move(mapped_pairs.second), tlen_limit);
                         // Record that we mapped a read.
                         reads_mapped_by_thread.at(omp_get_thread_num()) += 2;
                     }
+                    
+                    if (!minimizer_mapper.fragment_distr_is_finalized() && ambiguous_pair_buffer.size() >= MAX_BUFFERED_PAIRS) {
+                        // We risk running out of memory if we keep this up.
+                        cerr << "warning[vg::giraffe]: Encountered " << ambiguous_pair_buffer.size() << " ambiguously-paired reads before finding enough" << endl
+                             << "                      unambiguously-paired reads to learn fragment length distribution. Are you sure" << endl
+                             << "                      your reads are paired and your graph is not a hairball?" << endl;
+                        require_distribution_finalized();
+                    }
                 };
 
                 if (!gam_filename.empty()) {
@@ -1167,16 +1302,19 @@ int main_giraffe(int argc, char** argv) {
                     fastq_paired_interleaved_for_each_parallel_after_wait(fastq_filename_1, map_read_pair, distribution_is_ready);
                 }
 
-                //Now map all the ambiguous pairs
-                //TODO: What do we do if we haven't finalized the distribution?
-                if (!minimizer_mapper.fragment_distr_is_finalized()){
-                    cerr << "warning[vg::giraffe]: Finalizing fragment length distribution before reaching maximum sample size" << endl;
-                    minimizer_mapper.finalize_fragment_length_distr();
-                }
+                // Now map all the ambiguous pairs
+                // Make sure fragment length distribution is finalized first.
+                require_distribution_finalized();
                 for (pair<Alignment, Alignment>& alignment_pair : ambiguous_pair_buffer) {
 
                     auto mapped_pairs = minimizer_mapper.map_paired(alignment_pair.first, alignment_pair.second);
-                    alignment_emitter->emit_mapped_pair(std::move(mapped_pairs.first), std::move(mapped_pairs.second));
+                    // Work out whether it could be properly paired or not, if that is relevant.
+                    int64_t tlen_limit = 0;
+                    if (hts_output && minimizer_mapper.fragment_distr_is_finalized()) {
+                         tlen_limit = minimizer_mapper.get_fragment_length_mean() + 6 * minimizer_mapper.get_fragment_length_stdev();
+                    }
+                    // Emit the read
+                    alignment_emitter->emit_mapped_pair(std::move(mapped_pairs.first), std::move(mapped_pairs.second), tlen_limit);
                     // Record that we mapped a read.
                     reads_mapped_by_thread.at(omp_get_thread_num()) += 2;
                 }
@@ -1249,6 +1387,4 @@ int main_giraffe(int argc, char** argv) {
 }
 
 // Register subcommand
-static Subcommand vg_giraffe("giraffe", "Graph Alignment Format Fast Emitter", DEVELOPMENT, main_giraffe);
-
-
+static Subcommand vg_giraffe("giraffe", "fast haplotype-aware short read alignment", PIPELINE, 4, main_giraffe);
