@@ -3,14 +3,14 @@
 #include "../xg.hpp"
 #include "../utility.hpp"
 #include "../mapper.hpp"
-#include "../surjector.hpp"
-#include "../alignment_emitter.hpp"
+#include "../hts_alignment_emitter.hpp"
 #include <vg/io/stream.hpp>
 #include <vg/io/vpkg.hpp>
 #include <bdsg/overlays/overlay_helper.hpp>
 
 #include <unistd.h>
 #include <getopt.h>
+#include <chrono>
 
 using namespace vg;
 using namespace vg::subcommand;
@@ -85,6 +85,7 @@ void help_map(char** argv) {
          << "    -j, --output-json             output JSON rather than an alignment stream (helpful for debugging)" << endl
          << "    -%, --gaf                     output alignments in GAF format" << endl
          << "    --surject-to TYPE             surject the output into the graph's paths, writing TYPE := bam |sam | cram" << endl
+         << "    --ref-paths FILE              ordered list of paths in the graph, one per line or HTSlib .dict, for HTSLib @SQ headers" << endl
          << "    --buffer-size INT             buffer this many alignments together before outputting in GAM [512]" << endl
          << "    -X, --compare                 realign GAM input (-G), writing alignment with \"correct\" field set to overlap with input" << endl
          << "    -v, --refpos-table            for efficient testing output a table of name, chr, pos, mq, score" << endl
@@ -92,12 +93,15 @@ void help_map(char** argv) {
          << "    -M, --max-multimaps INT       produce up to INT alignments for each read [1]" << endl
          << "    -Q, --mq-max INT              cap the mapping quality at INT [60]" << endl
          << "    --exclude-unaligned           exclude reads with no alignment" << endl
-         << "    -D, --debug                   print debugging information about alignment to stderr" << endl;
+         << "    -D, --debug                   print debugging information about alignment to stderr" << endl
+         << "    --log-time                    print runtime to stderr" << endl;
 
 }
 
 int main_map(int argc, char** argv) {
 
+    std::chrono::time_point<std::chrono::system_clock> launch = std::chrono::system_clock::now();
+
     if (argc == 2) {
         help_map(argv);
         return 1;
@@ -106,6 +110,7 @@ int main_map(int argc, char** argv) {
     #define OPT_SCORE_MATRIX 1000
     #define OPT_RECOMBINATION_PENALTY 1001
     #define OPT_EXCLUDE_UNALIGNED 1002
+    #define OPT_REF_PATHS 1003
     string matrix_file_name;
     string seq;
     string qual;
@@ -122,6 +127,7 @@ int main_map(int argc, char** argv) {
     int max_multimaps = 1;
     int thread_count = 1;
     string output_format = "GAM";
+    string ref_paths_name;
     bool exclude_unaligned = false;
     bool debug = false;
     float min_score = 0;
@@ -182,7 +188,7 @@ int main_map(int argc, char** argv) {
     int max_sub_mem_recursion_depth = 2;
     bool xdrop_alignment = false;
     uint32_t max_gap_length = 40;
-    bool surject_subpath_global = true; // force full length alignment in mpmap surjection resolution
+    bool log_time = false;
 
     int c;
     optind = 2; // force optind past command positional argument
@@ -253,17 +259,19 @@ int main_map(int argc, char** argv) {
                 {"id-mq-weight", required_argument, 0, '7'},
                 {"refpos-table", no_argument, 0, 'v'},
                 {"surject-to", required_argument, 0, '5'},
+                {"ref-paths", required_argument, 0, OPT_REF_PATHS},
                 {"no-patch-aln", no_argument, 0, '8'},
                 {"drop-full-l-bonus", no_argument, 0, '2'},
                 {"unpaired-cost", required_argument, 0, 'S'},
                 {"max-gap-length", required_argument, 0, 1},
                 {"xdrop-alignment", no_argument, 0, 2},
                 {"gaf", no_argument, 0, '%'},
+                {"log-time", no_argument, 0, '^'},
                 {0, 0, 0, 0}
             };
 
         int option_index = 0;
-        c = getopt_long (argc, argv, "s:J:Q:d:x:g:1:T:N:R:c:M:t:G:jb:Kf:iw:P:Dk:Y:r:W:6H:Z:q:z:o:y:Au:B:I:S:l:e:C:V:O:L:a:n:E:X:UpF:m:7:v5:824:3:9:0:%",
+        c = getopt_long (argc, argv, "s:J:Q:d:x:g:1:T:N:R:c:M:t:G:jb:Kf:iw:P:Dk:Y:r:W:6H:Z:q:z:o:y:Au:B:I:S:l:e:C:V:O:L:a:n:E:X:UpF:m:7:v5:824:3:9:0:%^",
                          long_options, &option_index);
 
 
@@ -521,6 +529,10 @@ int main_map(int argc, char** argv) {
                 return 1;
             }
             break;
+            
+        case OPT_REF_PATHS:
+            ref_paths_name = optarg;
+            break;
 
         case '8':
             patch_alignments = false;
@@ -575,6 +587,9 @@ int main_map(int argc, char** argv) {
             break;
 
         case 'h':
+        case '^':
+            log_time = true;
+            break;
         case '?':
             /* getopt_long already printed an error message. */
             help_map(argv);
@@ -588,6 +603,14 @@ int main_map(int argc, char** argv) {
         }
     }
 
+    // Decide if we are outputting to an htslib format
+    bool hts_output = (output_format == "SAM" || output_format == "BAM" || output_format == "CRAM");
+
+    if (!ref_paths_name.empty() && !hts_output) {
+        cerr << "warning:[vg map] Reference path file (--ref-paths) is only used when output format (--surject-to) is SAM, BAM, or CRAM." << endl;
+        ref_paths_name = "";
+    }
+
     if (seq.empty() && read_file.empty() && hts_file.empty() && fastq1.empty() && gam_input.empty() && fasta_file.empty()) {
         cerr << "error:[vg map] A sequence or read file is required when mapping." << endl;
         return 1;
@@ -606,7 +629,7 @@ int main_map(int argc, char** argv) {
         return 1;
     }
     // note: still possible that hts file types don't have quality, but have to check the file to know
-
+    
     MappingQualityMethod mapping_quality_method = Approx;
 
     string file_name;
@@ -653,12 +676,16 @@ int main_map(int argc, char** argv) {
     
     // One of them may be used to provide haplotype scores
     haplo::ScoreProvider* haplo_score_provider = nullptr;
-
-    // We try opening the file, and then see if it worked
-    ifstream xg_stream(xg_name);
     
-    if(xg_stream) {
+    if(!xg_name.empty()) {
         // We have an xg index!
+
+        // We try opening the file, and then see if it worked
+        ifstream xg_stream(xg_name);
+        if (!xg_stream) {
+            cerr << "Error[vg map]: Unable to open xg file \"" << xg_name << "\"" << endl;
+            exit(1);
+        }
         
         // TODO: tell when the user asked for an XG vs. when we guessed one,
         // and error when the user asked for one and we can't find it.
@@ -718,71 +745,36 @@ int main_map(int argc, char** argv) {
     
     // When outputting single-ended alignments, we need an empty vector to pass around
     vector<Alignment> empty_alns;
-    
-    // if no paths were given take all of those in the index
-    set<string> path_names;
-    if ((output_format == "SAM" || output_format == "BAM" || output_format == "CRAM") && path_names.empty()) {
-        xgidx->for_each_path_handle([&](const path_handle_t& path_handle) {
-            path_names.insert(xgidx->get_path_name(path_handle));
-        });
+   
+    // Look up all the paths we might need to surject to.
+    vector<path_handle_t> paths;
+    if (hts_output) {
+        paths = get_sequence_dictionary(ref_paths_name, *xgidx);
     }
     
-    // If we need to do surjection, we will need a surjector. So set one up.
-    Surjector surjector(xgidx);
-
-    // Look up all the path info we need for the HTSlib header, in case we output to HTS format.
-    map<string, int64_t> path_length;
-    xgidx->for_each_path_handle([&](path_handle_t path_handle) {
-            path_length[xgidx->get_path_name(path_handle)] = xgidx->get_path_length(path_handle);
-        });
-
-    // Set up output to an emitter that will handle serialization
-    unique_ptr<AlignmentEmitter> alignment_emitter = get_alignment_emitter("-", output_format, path_length, thread_count, xgidx);
-
-    // TODO: Refactor the surjection code out of surject_main and into somewhere where we can just use it here!
-
-    auto surject_alignments = [&](const vector<Alignment>& alns1, const vector<Alignment>& alns2) {
-        
-        if (alns1.empty()) return;
-        vector<Alignment> surjects1, surjects2;
-        int tid = omp_get_thread_num();
-        for (auto& aln : alns1) {
-            // Surject each alignment of the first read in the pair and annotate with surjected path position
-            surjects1.push_back(surjector.surject(aln, path_names, surject_subpath_global));
-        }
-        
-        for (auto& aln : alns2) {
-            // Surject each alignment of the second read in the pair, if any, and annotate with surjected path position
-            surjects2.push_back(surjector.surject(aln, path_names, surject_subpath_global));
-        }
-        
-        if (surjects2.empty()) {
-            // Write out surjected single-end reads
-            alignment_emitter->emit_mapped_single(std::move(surjects1));
-        } else {
-            // Look up the paired end distribution stats for deciding if reads are propelry paired
-            auto& stats = mapper[omp_get_thread_num()]->frag_stats;
-            // Put a proper pair bound at 6 std devs.
-            // If distribution hasn't been computed yet, this comes out 0 and no bound is applied.
-            int64_t tlen_limit = stats.cached_fragment_length_mean + 6 * stats.cached_fragment_length_stdev;
-        
-            // Write out surjected paired-end reads
-            alignment_emitter->emit_mapped_pair(std::move(surjects1), std::move(surjects2), tlen_limit);
-        }
-    };
+    // Set up output to an emitter that will handle serialization and surjection
+    unique_ptr<vg::io::AlignmentEmitter> alignment_emitter = get_alignment_emitter("-", output_format, paths, thread_count, xgidx);
 
     // We have one function to dump alignments into
     auto output_alignments = [&](vector<Alignment>& alns1, vector<Alignment>& alns2) {
-        if (output_format == "SAM" || output_format == "BAM" || output_format == "CRAM") {
-            // Surject and emit, making sure to pass tlen limit for proper pairing if paired.
-            surject_alignments(alns1, alns2);
+        if (alns2.empty()) {
+            // Single-ended read
+            alignment_emitter->emit_mapped_single(std::move(alns1));
         } else {
-            // Just emit. No need for a tlen limit.
-            if (alns2.empty()) {
-                // Single-ended read
-                alignment_emitter->emit_mapped_single(std::move(alns1));
+            // Paired reads
+            if (hts_output) {
+                // We need a tlen limit for flags
+                
+                // Look up the paired end distribution stats for deciding if reads are propelry paired
+                auto& stats = mapper[omp_get_thread_num()]->frag_stats;
+                // Put a proper pair bound at 6 std devs.
+                // If distribution hasn't been computed yet, this comes out 0 and no bound is applied.
+                int64_t tlen_limit = stats.cached_fragment_length_mean + 6 * stats.cached_fragment_length_stdev;
+                
+                // Send the tlen limit when emitting
+                alignment_emitter->emit_mapped_pair(std::move(alns1), std::move(alns2), tlen_limit);
             } else {
-                // Paired reads
+                // No need for a tlen limit
                 alignment_emitter->emit_mapped_pair(std::move(alns1), std::move(alns2));
             }
         }
@@ -855,6 +847,9 @@ int main_map(int argc, char** argv) {
         m->patch_alignments = patch_alignments;
         mapper[i] = m;
     }
+    vector<size_t> reads_mapped_by_thread(thread_count, 0);
+
+    std::chrono::time_point<std::chrono::system_clock> init = std::chrono::system_clock::now();
 
     if (!seq.empty()) {
         int tid = omp_get_thread_num();
@@ -888,6 +883,7 @@ int main_map(int argc, char** argv) {
 
         // Output the alignments in the correct format, possibly surjecting.
         output_alignments(alignments, empty_alns);
+        reads_mapped_by_thread[tid] += 1;
     }
 
     if (!read_file.empty()) {
@@ -926,6 +922,7 @@ int main_map(int argc, char** argv) {
                     // Output the alignments in the correct format, possibly surjecting. 
                     output_alignments(alignments, empty_alns);
                 }
+                reads_mapped_by_thread[tid] += 1;
             }
         }
     }
@@ -955,6 +952,8 @@ int main_map(int argc, char** argv) {
                 }
                 // Output the alignments in the correct format, possibly surjecting.
                 output_alignments(alignments, empty_alns);
+
+                reads_mapped_by_thread[tid] += 1;
             }
         };
 #pragma omp parallel for
@@ -989,6 +988,8 @@ int main_map(int argc, char** argv) {
 
             // Output the alignments in JSON or protobuf as appropriate.
             output_alignments(alignments, empty_alns);
+
+            reads_mapped_by_thread[tid] += 1;
         };
         // run
         hts_for_each_parallel(hts_file, lambda);
@@ -1035,6 +1036,8 @@ int main_map(int argc, char** argv) {
                         our_mapper->imperfect_pairs_to_retry.clear();
                     }
                 }
+
+                reads_mapped_by_thread[omp_get_thread_num()] += 2;
             };
             fastq_paired_interleaved_for_each_parallel(fastq1, lambda);
 #pragma omp parallel
@@ -1067,6 +1070,7 @@ int main_map(int argc, char** argv) {
                                                                                 xdrop_alignment);
                         //cerr << "This is just before output_alignments" << alignment.DebugString() << endl;
                         output_alignments(alignments, empty_alns);
+                        reads_mapped_by_thread[tid] += 1;
                     };
             fastq_unpaired_for_each_parallel(fastq1, lambda);
         } else {
@@ -1108,6 +1112,8 @@ int main_map(int argc, char** argv) {
                         our_mapper->imperfect_pairs_to_retry.clear();
                     }
                 }
+
+                reads_mapped_by_thread[omp_get_thread_num()] += 2;
             };
             fastq_paired_two_files_for_each_parallel(fastq1, fastq2, lambda);
 #pragma omp parallel
@@ -1125,6 +1131,8 @@ int main_map(int argc, char** argv) {
                     output_func(p.first, p.second, alnp);
                 }
                 our_mapper->imperfect_pairs_to_retry.clear();
+
+                reads_mapped_by_thread[omp_get_thread_num()] += 2;
             }
         }
     }
@@ -1177,6 +1185,7 @@ int main_map(int argc, char** argv) {
                         our_mapper->imperfect_pairs_to_retry.clear();
                     }
                 }
+                reads_mapped_by_thread[omp_get_thread_num()] += 2;
             };
             vg::io::for_each_interleaved_pair_parallel(gam_in, lambda);
 #pragma omp parallel
@@ -1212,6 +1221,7 @@ int main_map(int argc, char** argv) {
                     alignment_set_distance_to_correct(alignments.front(), alignment);
                 }
                 output_alignments(alignments, empty_alns);
+                reads_mapped_by_thread[tid] += 1;
             };
             vg::io::for_each_parallel(gam_in, lambda);
         }
@@ -1231,6 +1241,22 @@ int main_map(int argc, char** argv) {
         delete haplo_score_provider;
         haplo_score_provider = nullptr;
     }
+    std::chrono::time_point<std::chrono::system_clock> end = std::chrono::system_clock::now();
+    std::chrono::duration<double> mapping_seconds = end - init;
+    std::chrono::duration<double> index_load_seconds = init - launch;
+
+    if (log_time){
+
+        size_t total_reads_mapped = 0;
+        for (auto& reads_mapped : reads_mapped_by_thread) {
+            total_reads_mapped += reads_mapped;
+        }
+    
+        double reads_per_second_per_thread = total_reads_mapped / (mapping_seconds.count() * thread_count);
+        cerr << "Index load time: " << index_load_seconds.count() << endl;
+        cerr << "Mapped " << total_reads_mapped << " reads" << endl;
+        cerr << "Mapping speed: " << reads_per_second_per_thread << " reads per second per thread" << endl; 
+    }
     
     cout.flush();
 
