@@ -220,15 +220,27 @@ int main_pack(int argc, char** argv) {
     }
 
     std::function<void(Alignment&)> lambda = [&packer,&min_mapq,&min_baseq](Alignment& aln) {
-            packer.add(aln, min_mapq, min_baseq);
-        };
+        packer.add(aln, min_mapq, min_baseq);
+    };
 
     if (!gam_in.empty()) {
         get_input_file(gam_in, [&](istream& in) {
                 vg::io::for_each_parallel(in, lambda, batch_size);
             });
     } else if (!gaf_in.empty()) {
-        gaf_unpaired_for_each_parallel(*graph, gaf_in, lambda, batch_size);
+        // we use this interface so we can ignore sequence, which takes a lot of time to parse
+        // and is unused by pack
+        function<size_t(nid_t)> node_to_length = [&graph](nid_t node_id) {
+            return graph->get_length(graph->get_handle(node_id));
+        };
+        function<string(nid_t, bool)> node_to_sequence = [&graph](nid_t node_id, bool is_reversed) {
+            return graph->get_sequence(graph->get_handle(node_id, is_reversed));
+        };
+
+        // computed batch size was tuned for GAM performance.  some small tests show that
+        // gaf benefits from a slightly larger one. 
+        vg::io::gaf_unpaired_for_each_parallel(node_to_length, record_edits ? node_to_sequence : nullptr,
+                                               gaf_in, lambda, batch_size * 4);
     }
 
     if (!packs_out.empty()) {
