@@ -8,7 +8,7 @@
 #include "subcommand.hpp"
 #include <gcsa/gcsa.h>
 #include <vg/io/stream.hpp>
-#include "../json2pb.h"
+#include "vg/io/json2pb.h"
 #include "../vg.hpp"
 #include <vg/vg.pb.h>
 #include "../filter.hpp"
@@ -585,6 +585,6 @@ else{
     return 0;
 }
 
-static Subcommand vg_sift("sift", "Filter Alignments by various metrics related to variant calling.", main_sift);
+static Subcommand vg_sift("sift", "Filter Alignments by various metrics related to variant calling.", DEPRECATED, main_sift);
 
 
