@@ -17,11 +17,6 @@
 #include "subcommand.hpp"
 #include "../algorithms/distance_to_head.hpp"
 #include "../algorithms/distance_to_tail.hpp"
-#include "../algorithms/find_tips.hpp"
-#include "../algorithms/is_acyclic.hpp"
-#include "../algorithms/strongly_connected_components.hpp"
-#include "../algorithms/weakly_connected_components.hpp"
-#include "../algorithms/copy_graph.hpp"
 #include "../handle.hpp"
 #include "../cactus_snarl_finder.hpp"
 
@@ -29,6 +24,12 @@
 #include "../statistics.hpp"
 #include "../genotypekit.hpp"
 
+#include "xg.hpp"
+#include "bdsg/packed_graph.hpp"
+#include "bdsg/hash_graph.hpp"
+#include "bdsg/odgi.hpp"
+#include "../io/converted_hash_graph.hpp"
+
 using namespace std;
 using namespace vg;
 using namespace vg::subcommand;
@@ -57,6 +58,9 @@ void help_stats(char** argv) {
          << "                          multiple allowed; limit comparison to those provided" << endl
          << "    -O, --overlap-all     print overlap table for the cartesian product of paths" << endl
          << "    -R, --snarls          print statistics for each snarl" << endl
+         << "    -F, --format          graph format from {VG-Protobuf, PackedGraph, HashGraph, ODGI, XG}. " <<
+        "Can't detect Protobuf if graph read from stdin" << endl
+         << "    -D, --degree-dist     print degree distribution of the graph." << endl
          << "    -v, --verbose         output longer reports" << endl;
 }
 
@@ -88,6 +92,8 @@ int main_stats(int argc, char** argv) {
     vector<string> paths_to_overlap;
     bool overlap_all_paths = false;
     bool snarl_stats = false;
+    bool format = false;
+    bool degree_dist = false;
 
     int c;
     optind = 2; // force optind past command positional argument
@@ -113,11 +119,13 @@ int main_stats(int argc, char** argv) {
             {"overlap", no_argument, 0, 'o'},
             {"overlap-all", no_argument, 0, 'O'},
             {"snarls", no_argument, 0, 'R'},
+            {"format", no_argument, 0, 'F'},
+            {"degree-dist", no_argument, 0, 'D'}, 
             {0, 0, 0, 0}
         };
 
         int option_index = 0;
-        c = getopt_long (argc, argv, "hzlsHTcdtn:NEa:vAro:OR",
+        c = getopt_long (argc, argv, "hzlsHTcdtn:NEa:vAro:ORFD",
                 long_options, &option_index);
 
         // Detect the end of the options.
@@ -202,6 +210,14 @@ int main_stats(int argc, char** argv) {
             verbose = true;
             break;
 
+        case 'F':
+            format = true;
+            break;
+
+        case 'D':
+            degree_dist = true;
+            break;
+
         case 'h':
         case '?':
             help_stats(argv);
@@ -214,11 +230,13 @@ int main_stats(int argc, char** argv) {
     }
 
     unique_ptr<PathHandleGraph> graph;
+    string graph_file_name;
     if (have_input_file(optind, argc, argv)) {
         // We have an (optional, because we can just process alignments) graph input file.
         // TODO: we can load any PathHandleGraph, but some operations still require a VG
         // In those cases, we convert back to vg::VG
-        graph = vg::io::VPKG::load_one<PathHandleGraph>(get_input_file_name(optind, argc, argv));
+        graph_file_name = get_input_file_name(optind, argc, argv);
+        graph = vg::io::VPKG::load_one<PathHandleGraph>(graph_file_name);
     }
     
     // We have function to make sure the graph was passed and complain if not
@@ -253,7 +271,7 @@ int main_stats(int argc, char** argv) {
 
     if (stats_heads) {
         require_graph();
-        vector<handle_t> heads = algorithms::head_nodes(graph.get());
+        vector<handle_t> heads = handlealgs::head_nodes(graph.get());
         cout << "heads" << "\t";
         for (auto& h : heads) {
             cout << graph->get_id(h) << " ";
@@ -263,7 +281,7 @@ int main_stats(int argc, char** argv) {
 
     if (stats_tails) {
         require_graph();
-        vector<handle_t> tails = algorithms::tail_nodes(graph.get());
+        vector<handle_t> tails = handlealgs::tail_nodes(graph.get());
         cout << "tails" << "\t";
         for (auto& t : tails) {
             cout << graph->get_id(t) << " ";
@@ -278,7 +296,7 @@ int main_stats(int argc, char** argv) {
         // but this isn't really explained.
         
         vector<pair<unordered_set<nid_t>, vector<handle_t>>> subgraphs_with_tips =
-            algorithms::weakly_connected_components_with_tips(graph.get());
+            handlealgs::weakly_connected_components_with_tips(graph.get());
         
         for (auto& subgraph_and_tips : subgraphs_with_tips) {
             // For each subgraph set and its inward tip handles
@@ -320,7 +338,7 @@ int main_stats(int argc, char** argv) {
 
     if (show_components) {
         require_graph();
-        for (auto& c : algorithms::strongly_connected_components(graph.get())) {
+        for (auto& c : handlealgs::strongly_connected_components(graph.get())) {
             for (auto& id : c) {
                 cout << id << ", ";
             }
@@ -330,7 +348,7 @@ int main_stats(int argc, char** argv) {
 
     if (is_acyclic) {
         require_graph();
-        if (algorithms::is_acyclic(graph.get())) {
+        if (handlealgs::is_acyclic(graph.get())) {
             cout << "acyclic" << endl;
         } else {
             cout << "cyclic" << endl;
@@ -355,6 +373,50 @@ int main_stats(int argc, char** argv) {
         }
     }
 
+    if (format) {
+        require_graph();
+        string format_string;
+        if (dynamic_cast<xg::XG*>(graph.get()) != nullptr) {
+            format_string = "XG";
+        } else if (dynamic_cast<bdsg::PackedGraph*>(graph.get()) != nullptr) {
+            format_string = "PackedGraph";
+        } else if (dynamic_cast<vg::io::ConvertedHashGraph*>(graph.get()) != nullptr) {
+            // Was Protobuf but we're using a HashGraph internally
+            format_string = "VG-Protobuf";
+        } else if (dynamic_cast<bdsg::HashGraph*>(graph.get()) != nullptr) {
+            format_string = "HashGraph";
+        } else if (dynamic_cast<bdsg::ODGI*>(graph.get()) != nullptr) {
+            format_string = "ODGI";
+        } else {
+            format_string = "Unknown";
+        }
+        cout << "format: " << format_string << endl;
+    }
+
+    if (degree_dist) {
+        require_graph();
+        // compute degrees
+        map<size_t, tuple<size_t, size_t, size_t, size_t>> degree_to_count;
+        graph->for_each_handle([&degree_to_count, &graph](handle_t handle) {
+                size_t left_degree = graph->get_degree(handle, true);
+                size_t right_degree = graph->get_degree(handle, false);
+                // update sides count
+                ++get<0>(degree_to_count[left_degree]);
+                ++get<0>(degree_to_count[right_degree]);
+                // update min count
+                ++get<1>(degree_to_count[std::min(left_degree, right_degree)]);
+                // update max count
+                ++get<2>(degree_to_count[std::max(left_degree, right_degree)]);
+                // update total count
+                ++get<3>(degree_to_count[left_degree + right_degree]);
+            });
+        // print degrees
+        cout << "Degree\tSides\tNodes(min)\tNodes(max)\tNodes(total)" << endl;
+        for (const auto& dg : degree_to_count) {
+            cout << dg.first << "\t" << get<0>(dg.second) << "\t" <<get<1>(dg.second) << "\t" << get<2>(dg.second) << "\t" << get<3>(dg.second) << endl;
+        }
+    }
+
     if (!paths_to_overlap.empty() || overlap_all_paths) {
         require_graph();
         
@@ -362,7 +424,7 @@ int main_stats(int argc, char** argv) {
         if (vg_graph == nullptr) {
             // TODO: This path overlap code can be handle-ified, and should be.
             vg_graph = new vg::VG();
-            algorithms::copy_path_handle_graph(graph.get(), vg_graph);
+            handlealgs::copy_path_handle_graph(graph.get(), vg_graph);
             // Give the unique_ptr ownership and delete the graph we loaded.
             graph.reset(vg_graph);
             // Make sure the paths are all synced up
