@@ -24,6 +24,24 @@ typedef vector<gbwt::node_type> exon_nodes_t;
 typedef vector<gbwt::size_type> thread_ids_t;
 
 
+/**
+ * Data structure that defines a transcript annotation.
+ */
+struct Exon {
+
+    /// Exon coordinates (start and end) on the chromosome/contig.
+    pair<int32_t, int32_t> coordinates;
+
+    /// Exon border node offsets (last position in upstream intron and
+    /// first position in downstream intron) on a variation graph. 
+    pair<uint32_t, uint32_t> border_offsets;
+
+    /// Exon border reference path steps (last position in upstream intron and
+    /// first position in downstream intron) on a variation graph. 
+    pair<step_handle_t, step_handle_t> border_steps;
+};
+
+
 /**
  * Data structure that defines a transcript annotation.
  */
@@ -41,12 +59,8 @@ struct Transcript {
     /// Length of chromosome/contig where transcript exist.
     const uint32_t chrom_length;
 
-    /// Exon coordinates (start and end) on the chromosome/contig.
-    vector<pair<int32_t, int32_t> > exons;
-
-    /// Exon border node positions (last position in upstream intron and
-    /// first position in downstream intron) on a variation graph. 
-    vector<pair<Position, Position> > exon_border_nodes;
+    /// Transcript exons.
+    vector<Exon> exons;
 
     Transcript(const string & name_in, const bool is_reverse_in, const string & chrom_in, const uint32_t & chrom_length_in) : name(name_in), is_reverse(is_reverse_in), chrom(chrom_in), chrom_length(chrom_length_in) {
 
@@ -202,10 +216,10 @@ class Transcriptome {
         bool _splice_graph_node_updated;
 
         /// Parse BED file of introns.
-        vector<Transcript> parse_introns(istream & intron_stream) const;
+        vector<Transcript> parse_introns(istream & intron_stream, const bdsg::PositionOverlay & graph_path_pos_overlay) const;
 
         /// Parse gtf/gff3 file of transcripts.
-        vector<Transcript> parse_transcripts(istream & transcript_stream) const;
+        vector<Transcript> parse_transcripts(istream & transcript_stream, const bdsg::PositionOverlay & graph_path_pos_overlay) const;
 
         /// Returns the mean node length of the graph
         float mean_node_length() const;
@@ -220,17 +234,17 @@ class Transcriptome {
 
         /// Constructs edited transcript paths from a set of 
         /// reference transcripts. 
-        list<EditedTranscriptPath> construct_edited_transcript_paths(const vector<Transcript> & transcripts) const;
+        list<EditedTranscriptPath> construct_edited_transcript_paths(const vector<Transcript> & transcripts, const bdsg::PositionOverlay & graph_path_pos_overlay) const;
 
         /// Threaded edited transcript path construction.
-        void construct_edited_transcript_paths_callback(list<EditedTranscriptPath> * edited_transcript_paths, mutex * edited_transcript_paths_mutex, const int32_t thread_idx, const vector<Transcript> & transcripts) const;
+        void construct_edited_transcript_paths_callback(list<EditedTranscriptPath> * edited_transcript_paths, mutex * edited_transcript_paths_mutex, const bdsg::PositionOverlay & graph_path_pos_overlay, const int32_t thread_idx, const vector<Transcript> & transcripts) const;
 
         /// Constructs transcript paths by projecting transcripts onto embedded paths in
         /// a variation graph and/or haplotypes in a GBWT index. 
-        void project_and_add_transcripts(const vector<Transcript> & transcripts, const gbwt::GBWT & haplotype_index, const float mean_node_length);
+        void project_and_add_transcripts(const vector<Transcript> & transcripts, const gbwt::GBWT & haplotype_index, const bdsg::PositionOverlay & graph_path_pos_overlay, const float mean_node_length);
 
         /// Threaded transcript projecting.
-        void project_and_add_transcripts_callback(const int32_t thread_idx, const vector<Transcript> & transcripts, const gbwt::GBWT & haplotype_index, const float mean_node_length);
+        void project_and_add_transcripts_callback(const int32_t thread_idx, const vector<Transcript> & transcripts, const gbwt::GBWT & haplotype_index, const bdsg::PositionOverlay & graph_path_pos_overlay, const float mean_node_length);
 
         /// Projects transcripts onto haplotypes in a GBWT index and returns resulting transcript paths.
         list<EditedTranscriptPath> project_transcript_gbwt(const Transcript & cur_transcript, const gbwt::GBWT & haplotype_index, const float mean_node_length) const;
@@ -240,7 +254,7 @@ class Transcriptome {
         vector<pair<exon_nodes_t, thread_ids_t> > get_exon_haplotypes(const vg::id_t start_node, const vg::id_t end_node, const gbwt::GBWT & haplotype_index, const int32_t expected_length) const;
 
         /// Projects transcripts onto embedded paths in a variation graph and returns resulting transcript paths.
-        list<EditedTranscriptPath> project_transcript_embedded(const Transcript & cur_transcript, const bool reference_only) const;
+        list<EditedTranscriptPath> project_transcript_embedded(const Transcript & cur_transcript, const bdsg::PositionOverlay & graph_path_pos_overlay, const bool reference_only) const;
 
         /// Adds new transcript paths to current set. Has argument to only add unique paths.
         void append_transcript_paths(list<CompletedTranscriptPath> * completed_transcript_path, list<CompletedTranscriptPath> * new_completed_transcript_paths, const bool add_unqiue_paths_only) const;
