@@ -8,7 +8,7 @@
 #include <iostream>
 #include <set>
 #include "catch.hpp"
-#include "../json2pb.h"
+#include "vg/io/json2pb.h"
 #include "../genotypekit.hpp"
 #include "../cactus_snarl_finder.hpp"
 #include "../genome_state.hpp"
