@@ -8,13 +8,15 @@
 
 #include "../utility.hpp"
 #include "../path.hpp"
-#include "../json2pb.h"
+#include "vg/io/json2pb.h"
 
 #include <vector>
 #include <sstream>
 #include <iostream>
 #include <regex>
 
+using namespace vg::io;
+
 namespace vg {
 namespace unittest {
 
