@@ -1,14 +1,21 @@
 #include "variant_adder.hpp"
 #include "mapper.hpp"
+#include "algorithms/prune.hpp"
 
 //#define debug
 
 namespace vg {
 
 using namespace std;
+using namespace vg::io;
 
-
-VariantAdder::VariantAdder(VG& graph) : graph(graph), sync(graph) {
+VariantAdder::VariantAdder(VG& graph) : graph(graph), sync([&](VG& g) -> VG& {
+        // Dice nodes in the graph for GCSA indexing *before* constructing the synchronizer.
+        g.dice_nodes(max_node_size);
+        return g;
+    }(this->graph)) {
+    
+    
     graph.paths.for_each_name([&](const string& name) {
         // Save the names of all the graph paths, so we don't need to lock the
         // graph to check them.
@@ -18,10 +25,6 @@ VariantAdder::VariantAdder(VG& graph) : graph(graph), sync(graph) {
     // Show progress if the graph does.
     show_progress = graph.show_progress;
     
-    // Make sure to dice nodes to 1024 or smaller, the max size that GCSA2
-    // supports, in case we need to GCSA-index part of the graph.
-    graph.dice_nodes(max_node_size);
-    
     // Configure the aligner to use a full length bonus
     aligner.full_length_bonus = 5;
 }
@@ -638,9 +641,9 @@ Alignment VariantAdder::smart_align(vg::VG& graph, pair<NodeSide, NodeSide> endp
         
         // Turn N/N subs into matches and score the alignments without end bonuses.
         align_ns(left_subgraph, aln_left);
-        aln_left.set_score(aligner.score_ungapped_alignment(aln_left, true));
+        aln_left.set_score(aligner.score_contiguous_alignment(aln_left, true));
         align_ns(right_subgraph, aln_right);
-        aln_right.set_score(aligner.score_ungapped_alignment(aln_right, true));
+        aln_right.set_score(aligner.score_contiguous_alignment(aln_right, true));
         
         
 #ifdef debug
@@ -689,7 +692,7 @@ Alignment VariantAdder::smart_align(vg::VG& graph, pair<NodeSide, NodeSide> endp
 
                 // Rescore with Ns as matches again
                 align_ns(left_subgraph, aln);
-                aln.set_score(aligner.score_ungapped_alignment(aln, true));
+                aln.set_score(aligner.score_contiguous_alignment(aln, true));
 
 #ifdef debug                
                 if (aligned_in_band) {
@@ -728,8 +731,11 @@ Alignment VariantAdder::smart_align(vg::VG& graph, pair<NodeSide, NodeSide> endp
                 if (edge_max) {
                     VG gcsa_graph = graph; // copy the graph
                     // remove complex components
-                    gcsa_graph.prune_complex_with_head_tail(kmer_size, edge_max);
-                    if (subgraph_prune) gcsa_graph.prune_short_subgraphs(subgraph_prune);
+                    algorithms::prune_complex_with_head_tail(gcsa_graph, kmer_size, edge_max);
+                    if (subgraph_prune) {
+                        algorithms::prune_short_subgraphs(gcsa_graph, subgraph_prune);
+                    }
+                        
                     // then index
 #ifdef debug
                     cerr << "\tGCSA index size: " << gcsa_graph.length() << " bp" << endl;
@@ -800,7 +806,7 @@ Alignment VariantAdder::smart_align(vg::VG& graph, pair<NodeSide, NodeSide> endp
                 
                 // Rescore with Ns as matches again
                 align_ns(left_subgraph, aln);
-                aln.set_score(aligner.score_ungapped_alignment(aln, true));
+                aln.set_score(aligner.score_contiguous_alignment(aln, true));
                 
 #ifdef debug
                 cerr << "\tScore: " << aln.score() << "/" << (to_align.size() * aligner.match * min_score_factor)
