@@ -1,6 +1,5 @@
 #include <cstdint>
 #include "variant_recall.hpp"
-#include "algorithms/topological_sort.hpp"
 #include "cactus_snarl_finder.hpp"
 #include "traversal_finder.hpp"
 #include "augment.hpp"
@@ -71,14 +70,14 @@ void genotype_svs(VG* graph,
     vg::io::for_each_interleaved_pair_parallel(gamstream, readfunc);
     vector<Translation> transls;
     if (refpath != ""){
-        augment(graph, direct_ins, &transls);
+        augment(graph, direct_ins, "GAM", &transls);
 
         XG xg_index;
         xg_index.from_path_handle_graph(*graph); // Index the graph so deconstruct can get path positions
         Deconstructor decon;
         CactusSnarlFinder finder(xg_index);
         SnarlManager snarl_manager = finder.find_snarls();
-        decon.deconstruct({refpath}, &xg_index, &snarl_manager, false, 1);
+        decon.deconstruct({refpath}, &xg_index, &snarl_manager, false, 1, false);
     }
     direct_ins.clear();
 
@@ -123,7 +122,7 @@ void variant_recall(VG* graph,
                                vcflib::VariantCallFile* vars,
                                FastaReference* ref_genome,
                                vector<FastaReference*> insertions,
-                               string gamfile, bool isIndex){
+                               string gamfile){
     // Store variant->name index
     map<string, vcflib::Variant> hash_to_var;
     set<int64_t> variant_nodes;
@@ -348,7 +347,14 @@ void variant_recall(VG* graph,
             
     };
     // open our gam, count our reads, close our gam.
-    if (!isIndex){
+    if (use_snarls){
+        ifstream gamstream(gamfile);
+        if (gamstream.good()){
+            vg::io::for_each(gamstream, count_traversal_supports);
+        }
+        gamstream.close();
+    }
+    else{
         ifstream gamstream(gamfile);
         if (gamstream.good()){
             vg::io::for_each(gamstream, incr);
@@ -359,19 +365,6 @@ void variant_recall(VG* graph,
         }
         gamstream.close();
     }
-    else if (use_snarls && !isIndex){
-        ifstream gamstream(gamfile);
-        if (gamstream.good()){
-            vg::io::for_each(gamstream, count_traversal_supports);
-        }
-        gamstream.close();
-    }
-    else{
-        Index gamindex;
-        gamindex.open_read_only(gamfile);
-        vector<nid_t> vn(variant_nodes.begin(), variant_nodes.end());
-        gamindex.for_alignment_to_nodes(vn, index_incr);
-    }
 
 
     std::function<long double(int64_t)> fac = [](int64_t t){
