@@ -573,11 +573,6 @@ public:
     /// Remove duplicated nodes and edges.
     void remove_duplicates(void);
 
-    /// Limit the local complexity of the graph, connecting pruned components to a head and tail node
-    /// depending on the direction which we come into the node when the edge_max is passed.
-    void prune_complex_paths(int length, int edge_max, Node* head_node, Node* tail_node);
-    void prune_short_subgraphs(size_t min_size);
-
     /// Send chunked graphs to a function that will write them somewhere.
     /// Used to internally implement saving to many destinations.
     /// Graph will be serialized in internal storage order.
@@ -672,14 +667,15 @@ public:
               bool break_at_ends = false);
 
     /// Streaming version of above.  Instead of reading a list of paths into memory
-    /// all at once, a stream is used to go one-by-one.  Instead of an option to
-    /// updtate the in-memory list, an optional output stream is used
+    /// all at once, a file stream is opened from the given path and used to go one-by-one.
+     /// Instead of an option to
+    /// updtate the in-memory list, an optional output path for the paths is used
     ///
     /// todo: duplicate less code between the two versions. 
-    void edit(istream& paths_to_add,
+    void edit(const string& paths_to_add_path,
               vector<Translation>* out_translations = nullptr,
               bool save_paths = false,
-              ostream* out_path_stream = nullptr,
+              const string& out_gam_path = "",
               bool break_at_ends = false,
               bool remove_softclips = false);
     
@@ -1008,6 +1004,7 @@ public:
                 bool annotate_paths = false,
                 bool show_mappings = false,
                 bool simple_mode = false,
+                bool noseq_mode = false,
                 bool invert_edge_ports = false,
                 bool color_variants = false,
                 bool ultrabubble_labeling = false,
@@ -1211,15 +1208,6 @@ public:
     void node_starts_in_path(list<NodeTraversal>& path,
                              map<NodeTraversal*, int>& node_start);
 
-    /// Take all nodes that would introduce paths of > edge_max edge crossings, remove them, and link their neighbors to
-    /// head_node or tail_node depending on which direction the path extension was stopped.
-    /// Optionally preserve edges on the embedded paths.
-    /// For pruning graph prior to indexing with gcsa2.
-    void prune_complex(int path_length, int edge_max, Node* head_node, Node* tail_node);
-    /// Wrap the graph with heads and tails before doing the prune.
-    /// Utility function for preparing for indexing.
-    void prune_complex_with_head_tail(int path_length, int edge_max);
-
 private:
     /// Call the given function on each kmer. If parallel is specified, goes
     /// through nodes one per thread. If node is not null, looks only at kmers of
@@ -1266,8 +1254,6 @@ public:
     /// graph will result in some reads from the global reverse strand.
     Alignment random_read(size_t read_len, mt19937& rng, nid_t min_id, nid_t max_id, bool either_strand);
 
-    /// Find subgraphs.
-    void disjoint_subgraphs(list<VG>& subgraphs);
     /// Get the head nodes (nodes with edges only to their right sides). These are required to be oriented forward.
     void head_nodes(vector<Node*>& nodes);
     /// Get the head nodes (nodes with edges only to their right sides). These are required to be oriented forward.
